/**
 * \file mgui/form.h
 * \brief Definitions for MGUI::FORM interface class.
 *
 * \if NODOC
 * $Id: form.h_v 1.15 2013/01/24 21:41:44 mju Exp $
 *
 * $Log: form.h_v $
 * Revision 1.15  2013/01/24 21:41:44  mju
 * Add form_composite::getDimensions.
 *
 * Revision 1.4  2002/09/03 22:37:27  dwilliss
 * Added virtual method to support SML hooks
 *
 * Revision 1.1  2001/11/30 17:27:32  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORM_H
#define	INC_MGUI_FORM_H

#ifndef INC_MGUI_LAYOUT_H
#include <mgui/layout.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class SML_GUI_INTERFACE_BASE;	// Hook for the SML interface to MGUI controls
#endif


namespace MGUI {

//=====================================================================================================================

//! Base interface class for graphical user interface controls and forms.
class FORM {
	public:

		//! Destructor.
		virtual ~FORM () = 0;

		//! Determine if form has been created.
		//! @return true if created, false if not.
		virtual bool IsCreated () const = 0;

		//! Set whether mouse or keyboard input to form is allowed.
		virtual void SetEnabled (
			bool enabled = true
			) = 0;

		//! Set focus to first interactive control in form.
		//! If calling for a DLGSHELL should be done in v_OnOpen, cannot
		//! be done in v_CreateContent.
		void SetFocus () const
			{ v_SetFocus(); }

		//! Set whether form is visible or not.
		virtual void SetVisible (
			bool visible = true
			) = 0;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Function private to SML implementation of MGUI classes.
		//! Used to prevent the need for ugly multiple inheritance.
		virtual SML_GUI_INTERFACE_BASE* GetSMLInterface ();
		#endif // GENERATING_DOXYGEN_OUTPUT

	private:

		//! Set focus to first interactive control in form.
		//! Default implementation does nothing.
		virtual void v_SetFocus () const;

	};

//=====================================================================================================================

//! Base class for composite forms.
class FORM_COMPOSITE : public MGUI::FORM {
	public:

		//! Constructor.
		FORM_COMPOSITE ();

		//! Destructor.
		virtual ~FORM_COMPOSITE () = 0;

		//! Get suitable parent for dialog launched from form.
		MDLGPARENT GetDlgParent () const
			{ return (m_pane.GetDlgParent()); }

		//! Get current form dimensions.
		MGUI::POINT GetDimensions () const
			{ return (m_pane.GetDimensions()); }

		//! Determine if form has been created.
		//! @return true if created, false if not.
		virtual bool IsCreated () const;

		//! Determine if form is visible.
		//! @return true if visible, false if not.
		//! Does not check if form is contained in a non-visible parent form.
		bool IsVisible () const
			{ return (m_pane.IsVisible()); }

		//! Set whether mouse or keyboard input to form is allowed.
		virtual void SetEnabled (
			bool enabled = true				//!< Whether enabled
			);

		//! Set whether form is visible or not.
		virtual void SetVisible (
			bool visible = true				//!< Whether visible
			);

	protected:

		//! Create layout pane.
		//! Use GetPane() to retrieve reference to pane.
		void CreatePane (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< 'Parent' pane
			MGUI::LAYOUT_ORIENTATION orientation,	//!< Pane orientation
			MGUI::LAYOUT_SIZEALIGN sizealign,		//!< Sizing and alignment
			int ChildSpacing = 4,						//!< Spacing between 'children' of pane
			int ExtraBorder = 0							//!< Extra border around pane, in addition to ChildSpacing
			) { m_pane.Create(ParentPane,orientation,sizealign,ChildSpacing,ExtraBorder); }

		//! Get layout pane.
		MGUI::LAYOUT_PANE& GetPane ()
			{ return (m_pane); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LAYOUT_PANE m_pane;

		FORM_COMPOSITE (const FORM_COMPOSITE&);
		FORM_COMPOSITE& operator= (const FORM_COMPOSITE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORM_H
