/**
 *	\file mgui/formeditcolors.h
 *	\brief MGUI::FORM_EDIT_COLORS class definitions
 *
 * \if NODOC
 * $Id: formeditcolors.h_v 1.8 2011/07/18 18:55:42 mju Exp $
 *
 * $Log: formeditcolors.h_v $
 * Revision 1.8  2011/07/18 18:55:42  mju
 * Add get/setItemID.
 *
 * Revision 1.7  2010/06/02 21:25:57  mju
 * Add redrawDisable/Enable.
 *
 * Revision 1.6  2010/06/02 20:31:34  mju
 * Add style_WidthInPixels and explicit scrollbar styles.
 *
 * Revision 1.5  2010/06/02 19:58:17  mju
 * Define enum ops.
 *
 * Revision 1.4  2010/06/02 18:00:11  mju
 * Use color::capability and make internal implementation private.
 * Change create to use style enum and other parms.
 *
 * Revision 1.3  2010/06/02 14:09:10  mju
 * Add typedef for delegate.
 *
 * Revision 1.2  2007/10/31 16:20:17  mju
 * Don't use grid template.
 *
 * Revision 1.1  2007/10/31 15:04:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMEDITCOLORS_H
#define	INC_MGUI_FORMEDITCOLORS_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

namespace MGUI {

//! Form for color edit list.
//! This form includes a list showing color name and sample,
//! and a set of controls to "edit" the selected color.
class FORM_EDIT_COLORS : public MGUI::FORM_COMPOSITE {
	public:

		typedef FastDelegate<void(int,const COLOR&)> DELEGATE_ONCHANGEITEMCOLOR;		//!< Delegate parameter: itemidx, color

		//! Form style.
		enum STYLE {
			STYLE_ListTop =			0x00,			//!< List at top (default)
			STYLE_ListBottom =		0x01,			//!< List at bottom
			STYLE_ListLeft =			0x02,			//!< List to left of palette
			STYLE_ListRight =			0x04,			//!< List to right of palette

			STYLE_ScrollAuto =		0x00,			//!< Determine whether to show scrollbars when Create() based on number of items and dimensions
			STYLE_ScrollNoH =			0x10,			//!< No horizontal scrollbar, always used if single-column mode
			STYLE_ScrollNoV =			0x20,			//!< No vertical scrollbar, typically for multi-column mode or if few fixed items
			STYLE_ScrollNone =		0x30,			//!< No scrollbars

			STYLE_WidthInPixels =	0x40,			//!< List width specified in pixels

			STYLE_Default =			STYLE_ListTop | STYLE_ScrollAuto
			};

		//! Constructor.
		FORM_EDIT_COLORS ();

		//! Destructor.
		virtual ~FORM_EDIT_COLORS ();

		//! Add item to list.
		//! @return Item index for use when retrieve color.
		int AddItem (
			const MISTRING& name,
			const COLOR& color,
			COLOR::CAPABILITY capability = COLOR::CAPABILITY_AsPrevious
			);

		//! Add item to list.
		//! @return Item index for use when retrieve color.
		int AddItem (
			const MISTRING& name,
			UINT32 ColorRef,
			COLOR::CAPABILITY capability = COLOR::CAPABILITY_AsPrevious
			);

		//! Create the form.
		//! Note that if transparency on/off or percentage controls are required for any
		//! item in the list the desired capability must be specified when creating the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane for form
			COLOR::CAPABILITY capability,				//!< Overall color edit capabilities
			int listrows,									//!< Initial number of visible rows in color list
			int listwidth = -1,							//!< Initial list width, -1 to compute based on items already added
			STYLE style = STYLE_Default,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand
			);

		//! Delete all items from list.
		void DeleteAllItems ();

		//! Get color for specified item.
		//! @return Reference to color
		const COLOR& GetItemColor (
			int itemidx								//!< Item index (from AddItem())
			) const;

		//! Get color reference for specified item.
		//! @return Color reference
		UINT32 GetItemColorRef (
			int itemidx								//!< Item index (from AddItem())
			) const;

		//! Get ID from index.
		MGUI::ID GetItemID (
			int itemidx
			) const;

		//! Disable redraw of form.
		void RedrawDisable ();

		//! Enable redraw of form.
		void RedrawEnable ();

		//! Set delegate to call when item color changed.
		void SetDelegateOnChangeItemColor (
			DELEGATE_ONCHANGEITEMCOLOR delegate		//!< Delegate parameter: itemidx, color
			);

		//! Set color for specified item
		void SetItemColor (
			int itemidx,							//!< Item index from AddItem()
			const COLOR& color,					//!< New color to set
			bool notify = true					//!< Call OnChangeItemColor() if 'true' and color changed
			);

		//! Set ID for index.
		void SetItemID (
			int itemidx,
			MGUI::ID id
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_EDIT_COLORS::STYLE)
#endif


}	// End namespace MGUI

#endif	// INC_MGUI_FORMEDITCOLORS_H
