/**
 * \file mgui/formfilelist.h
 * \brief MGUI::FORM_FILE_LIST class definitions
 *
 * \if NODOC
 * $Id: formfilelist.h_v 1.10 2011/11/29 22:21:14 mju Exp $
 *
 * $Log: formfilelist.h_v $
 * Revision 1.10  2011/11/29 22:21:14  mju
 * Remove redundant hdr.
 *
 * Revision 1.9  2009/04/15 16:18:21  bcolombe
 * Added FLAGS and FLAG_SelectFolders
 *
 * Revision 1.7  2006/09/25 17:29:55  TURKER
 * Added SetValue function
 *
 * Revision 1.4  2006/08/24 22:56:52  dwilliss
 * Added method to set extension
 *
 * Revision 1.1  2006/02/23 21:24:39  ldyas
 * Initial revision
 * \endif
**/

#ifndef INC_MGUI_FORMFILELIST_H
#define INC_MGUI_FORMFILELIST_H

#ifndef	INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_MI32_FILEPATHLIST_H
#include <mi32/filepathlist.h>
#endif

#ifndef	INC_MI32_STRUTF8_H
#include <mi32/strutf8.h>
#endif

namespace MGUI {
	
//===================================================================================================================
//! Form containing basic object selection list with "select", "remove", and "remove all" buttons.
class FORM_FILE_LIST : public MGUI::FORM_COMPOSITE {
	public:
		enum FLAGS {
			FLAG_Default =			0x0000,
			FLAG_SelectFolders =	0x0001,		//!< Prompt for folder selection instead of file
			};
	
		//! Constructor.
		FORM_FILE_LIST (
			);

		//! Destructor.
		virtual ~FORM_FILE_LIST (
			);
			
		//! Clears out all of the selected objects
		void Clear (
			bool notify							//!< Send notifications via virtual methods
			);

		//! Create object list form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			FLAGS flags = FLAG_Default
			);

		//! Get the current object, will return an empty OBJITEM if there is nothing in the list
		const FILEPATH& GetCurrentFile (
			) const;
			
		//! Get the list of objects in the list			
		const FILEPATHLIST& GetFileList (
			) const { return (m_FilePathList); }

		//! Is the list empty?			
		bool IsEmpty (
			) const { return (m_FilePathList.IsEmpty()); }

		//! Select objects via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify							//!< Send notifications via virtual methods
			);

		//! Sets the current file list
		ERRVALUE SetValue (							
			const FILEPATHLIST& PathList, //!< List to set from
			bool notify           //!< Send notifications via virtual methods
			);
			
		//! Set delegate to call for any change to objects in list.
		void SetDelegateOnListChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnListChanged = delegate; }

		//! Set delegate to call for each object added to list.
		void SetDelegateOnFileAdded (
			FastDelegate<void(const FILEPATH&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnFileAdded = delegate; }

		//! Set delegate to call when single object removed from list.
		void SetDelegateOnFileRemoved (
			FastDelegate<void(INT32)> delegate								//!< Delegate parameter: item index in list
			) { m_DelegateOnFileRemoved = delegate; }

		//! Set delegate to call when object selected (highlighted) in list.
		void SetDelegateOnFileSelected (
			FastDelegate<void(const FILEPATH&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnFileSelected = delegate; }

		//! Set delegate to call when objects added to list.
		void SetDelegateOnFilesAdded (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnFilesAdded = delegate; }

		//! Set delegate to call when all objects removed from list.
		void SetDelegateOnFilesAllRemoved (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnFilesAllRemoved = delegate; }

		//! Set the valid file extensions.
		//! String should contain the extension(s) without a dot.
		//! Multiple extensions can be seperated with a comma.
		void SetValidExtensions (
			const char* extns
			);

	protected:

		//! Called for any change to objects in list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnListChanged ();

		//! Called for each object added to list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnFileAdded (const FILEPATH& Filepath, INT32 ItemNum);

		//! Called when single object removed from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnFileRemoved (INT32 ItemNum);

		//! Called when object selected (highlighted) from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnFileSelected (const FILEPATH& Filepath, INT32 ItemNum);

		//! Called when objects added to list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnFilesAdded ();

		//! Called when all objects removed from list.
		//! Default implementation invokes corresponding delegate.
		//! If override call when/if needed to invoke delegate.
		virtual void v_OnFilesAllRemoved ();
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FLAGS m_flags;
		static FILEPATH *s_pNullItem;
		DELEGATE_VOID_NOPARMS m_DelegateOnListChanged;
		FastDelegate<void(const FILEPATH&, INT32)> m_DelegateOnFileAdded;
		FastDelegate<void(INT32)> m_DelegateOnFileRemoved;
		FastDelegate<void(const FILEPATH&, INT32)> m_DelegateOnFileSelected;
		DELEGATE_VOID_NOPARMS m_DelegateOnFilesAdded;
		DELEGATE_VOID_NOPARMS m_DelegateOnFilesAllRemoved;
		MGUI::CTRL_LISTBOX m_ListBox;
		MGUI::CTRL_PUSHBUTTON m_BtnSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnRemove;
		MGUI::CTRL_PUSHBUTTON m_BtnRemoveAll;
		FILEPATHLIST m_FilePathList;
		MISTRING m_PromptStr;
		STRUTF8 m_Extns;

		void OnListSelection ();
		void OnBtnRemove ();
		void OnBtnRemoveAll ();
		void OnBtnSelect ();

		FORM_FILE_LIST (const FORM_FILE_LIST&);
		FORM_FILE_LIST& operator= (const FORM_FILE_LIST&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
}		// End of MGUI namespace

#endif	//INC_MGUI_FORMFILELIST_H
