/**
 * \file mgui/statusbar.h
 * \brief MGUI::CTRL_STATUSBAR definition.
 *
 * \if NODOC
 * $Id: statusbar.h_v 1.11 2012/03/26 19:56:11 mju Exp $
 *
 * $Log: statusbar.h_v $
 * Revision 1.11  2012/03/26 19:56:11  mju
 * Add setPaneToolTipString.
 *
 * Revision 1.10  2012/01/27 16:31:18  mju
 * Use mgui::delegateontooltiprequest.
 *
 * Revision 1.9  2011/03/11 18:53:32  mju
 * Support tooltip for pane.
 *
 * Revision 1.8  2011/03/10 22:51:42  mju
 * Make internals private to implementation.
 *
 * Revision 1.7  2009/03/10 15:43:56  mju
 * Add clearPaneElapsedTime.
 *
 * Revision 1.6  2008/01/22 14:49:12  mju
 * Add xcb_destroy.
 *
 * Revision 1.5  2008/01/10 15:54:54  mju
 * Keep icon image list so can use when setPaneIcon.
 *
 * Revision 1.4  2007/12/28 18:53:53  mju
 * Add setPaneVisible.
 *
 * Revision 1.3  2007/11/16 18:51:41  scowan
 * Added timer for multi-thrreading work.
 *
 * Revision 1.2  2007/11/12 14:07:11  mju
 * Don't ref widget for mfc.
 *
 * Revision 1.1  2007/11/09 18:35:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_STATUSBAR_H
#define	INC_MGUI_STATUSBAR_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MI32_MISTATUS_H
#include <mi32/mistatus.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct MTIMER;
#endif


namespace MGUI {

//===================================================================================================================
//! StatusBar control 
//!
//! This control is not dynamically positioned the same as other MGUI::CTRLs because the underlying
//! implementation may do its own positioning in some environments.  When adding a status bar to an
//! MGUI::DLGSHELL you must use MGUI::DLGSHELL::SetStatusBar() in the dialog subclass's implementation
//! of v_CreateContent() to allow for the status bar in dynamic layout.
//! Logging is turned off by default, use SetLogEnabled() to enable if desired.
class CTRL_STATUSBAR : public MGUI::CTRL, public MISTATUSLOGGER {
	public:

		//! Status bar style
		enum STYLE {
			STYLE_Default =			0,
			STYLE_HasCancelButton =	0x01,		//!< Include cancel button
			STYLE_NoProgressBar =	0x02,		//!< Don't include progress bar indicator
			STYLE_NoElapsedTime =	0x04,		//!< Don't include indicator for elapsed time
			STYLE_EnableToolTips =	0x08,
			STYLE_EnableThreading =	0x10,		//!< Enable the status bar to handle status from multiple threads
			};

		enum PANEUSAGE {
			PANEUSAGE_Main,						//!< Main status message
			PANEUSAGE_ElapsedTime,				//!< Elapsed time
			PANEUSAGE_ProgressBar,				//!< Progress bar for status
			PANEUSAGE_Icon,						//!< Pane contains icon
			PANEUSAGE_Text,						//!< Pane contains text
			PANEUSAGE_Mixed,						//!< Both icon and text
			};

		//! Pane styles.
		enum PANESTYLE {
			PANESTYLE_Normal			= 0x00,
			PANESTYLE_NoSeparator	= 0x01,	//!< Don't separate from previous pane
			PANESTYLE_Stretch			= 0x02,	//!< Stretch pane relative to other "stretchable" panes
			};

		//! Constructor.
		CTRL_STATUSBAR ();

		//! Destructor.
		virtual ~CTRL_STATUSBAR ();

		//! Add "indicator pane" to status bar.
		//! @return Pane index, (might not be consecutive)
		//! If used, must be done before calling Create().
		//!
		//! If more than one pane has PANESTYLE_Stretch, their relative sizes will be based on the
		//! relative values of cxWidth.  For example, if pane 0 has a cxWidth of 100 and pane 1 has 50, then pane 0
		//! will be allocated twice as much of the available space as pane 1.
		int AddPane (
			PANEUSAGE usage,							//!< Pane usage, only one pane may have USAGE_Main, ProgressBar or ElapsedTime specified
			PANESTYLE style,							//!< Pane style
			int cxWidth,								//!< Pane width in pixels
			ID id = 0									//!< Optional pane ID
			);

		//! Attach button to use for cancelling operation.
		//! Delegate for button will be set.
		//! Button must have been created.
		void AttachCancelButton (
			CTRL_PUSHBUTTON& button
			);

		//! Attach toolbar button to use for cancelling operation.
		//! Delegate for toolbar button will be set.
		//! Toolbar must have been created.
		void AttachCancelButton (
			CTRL_TOOLBAR& toolbar,			//!< Toolbar
			ID id									//!< Id of toolbar button
			);

	#ifdef X_NATIVE
		DEPRECATED void AttachCancelButtonWidget (
			Widget widget
			);
	#endif

		//! Clear progress bar.
		void ClearBar (
			);

		//! Clear text or icon in specified pane.
		void ClearPane (
			int PaneIndex
			);

		//! Clear "elapsed time" pane (does not reset timer itself).
		void ClearPaneElapsedTime (
			);

		//! Clear text in main pane.
		void ClearText (
			);

		//! Create control.
		//! If specified style requests predefined panes that have not already been added via AddPane, they
		//! will be added to the right of any existing panes automatically.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,			//!< Parent pane
			STYLE style = STYLE_Default
			);

		//! Set delegate for tooltip request on pane.
		void SetPaneDelegateOnToolTipRequest (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			DELEGATE_ONTOOLTIPREQUEST delegate
			);
		
		//! Set whether pane shows as "enabled".
		void SetPaneEnabled (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			bool enabled								//!< True if "enabled", false if not
			);

		//! Set icon for specified pane.
		void SetPaneIcon (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			ICONID iconid								//!< Icon ID
			);

		//! Set text for specified pane.
		void SetPaneText (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			const MISTRING& string					//!< Text to set
			);

		//! Set string for tooltip on pane.
		void SetPaneToolTipString (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			const MISTRING& string					//!< String for tooltip
			);
		
		//! Set whether pane is visible.
		void SetPaneVisible (
			int PaneIndex,								//!< Pane index, 0 indicates main text pane
			bool visible								//!< True to show, false to not
			);

		//! Set text for "main" pane.
		//! Done automatically when create MISTATUSCONTEXT instance.
		void SetText (
			const MISTRING& string					//!< Text to set
			);

		//! Set text for first pane to error message.
		void SetTextError (
			ERRVALUE err
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef CTRL BASECLASS;
		typedef CTRL_STATUSBAR THISCLASS;

		class PRIV;
		PRIV *m_pPriv;

		void OnToolBarCancel (MGUI::ID, bool);

		// CTRL overrides.
		virtual void v_OnDestroyWidget ();

		// MISTATUSMANAGER/LOGGER overrides.
		virtual CONTEXT_PRIV* v_ContextPrivCreate (MISTATUSCONTEXT& Context);
		virtual bool v_IsCancelled ();
		virtual void v_ReportMessage (MISTATUSCONTEXT& context, const MISTRING& msgstr);
		virtual void v_ReportProgress (MISTATUSCONTEXT& context, double pctdone);
		virtual void v_ReportTimerValue (const MTIMER& timer);
		virtual void v_SetCancelEnabled (bool enabled);
		virtual void v_UpdateUI ();

		CTRL_STATUSBAR (const CTRL_STATUSBAR&);
		CTRL_STATUSBAR& operator= (const CTRL_STATUSBAR&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_STATUSBAR::STYLE)
DEFINE_ENUM_OP_BITWISE(CTRL_STATUSBAR::PANESTYLE)
#endif

//===================================================================================================================
}	// End namespace MGUI


#endif	// INC_MGUI_STATUSBAR_H
