/**
 * \file mi32/contrastparm.h
 * \brief Contrast parameter container (CONTRASTPARM).
 *
 * \if NODOC
 * $Id: contrastparm.h_v 1.18 2012/11/06 15:52:39 mju Exp $
 *
 * $Log: contrastparm.h_v $
 * Revision 1.18  2012/11/06 15:52:39  mju
 * Add hyperbolic tangent method.
 *
 * Revision 1.17  2012/06/08 14:48:34  mju
 * Add getMethodTextID.
 *
 * Revision 1.16  2012/06/05 21:34:06  mju
 * Add imageComponent member and accessor methods.
 *
 * Revision 1.15  2011/12/27 14:13:46  mju
 * Remove deprecated methods and redundant hdr.
 *
 * Revision 1.14  2011/03/09 15:29:02  mju
 * Add clear method.
 *
 * Revision 1.13  2010/06/17 19:58:17  mju
 * Add DetachTable.
 *
 * Revision 1.12  2008/06/13 20:24:28  mju
 * Add binInterval to setTableBinRange and return error.
 * Return error from setTable.
 *
 * Revision 1.11  2008/04/22 22:46:32  bcolombe
 * Added method for setting table bin range
 *
 * Revision 1.10  2007/09/13 21:47:44  mju
 * Add support for percentage input range.
 *
 * Revision 1.9  2007/09/06 21:15:42  mju
 * Add serialGetItemDef.
 *
 * Revision 1.7  2007/03/26 15:48:40  mju
 * Add setOutputMedian/StdDev...
 * Change "mean" to "median", with deprecated version of previous methods.
 *
 * Revision 1.6  2007/03/26 13:29:48  mju
 * Add setInputStretchMin/Max.
 *
 * Revision 1.5  2007/03/22 21:31:35  mju
 * Add clearTable.
 *
 * Revision 1.4  2007/02/27 14:54:25  mju
 * Add hasTable method.
 *
 * Revision 1.1  2006/11/09 16:44:50  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_CONTRASTPARM_H
#define	INC_MI32_CONTRASTPARM_H

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif

#ifndef	INC_MI32_NUMTYPE_H
	#include <mi32/numtype.h>
#endif

#ifdef RVCSYSDLL
	#define LIBEXPORT MI_DLLEXPORT
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class STAT_UNIVARIATE;
class SERIALIZERITEM;
#endif

//! Contrast parameters.
//	 If changed must rebuild DLL and all dependent executables.
class CLASSEXPORT CONTRASTPARM {
	public:

		//! Contrast table types.
		// Values are stored in files and may not change.
		enum METHOD {
			METHOD_None =					0,		//!< No contrasting
			METHOD_Linear =				1,		//!< Linear stretch between limits
			METHOD_Normalize =			2,		//!< Normalize, uses median, stdevl, stdevr to determine curve
			METHOD_Equalize =				3,		//!< Equalization
			METHOD_Exponential =			4,		//!< Exponential using power member
//			METHOD_PiecewiseLinear =	5,		//!< Piecewise linear
//			METHOD_Threshold =			6,		//!< Thresholding
			METHOD_SavedTable =			7,		//!< Previously saved table
											// 8 skipped to avoid conflict with legacy code
			METHOD_Logarithmic =			9,		//!< Logarithmic
			METHOD_Constant =				10,
			METHOD_HyperbolicTangent =	11,	//!< Hyperbolic Tangent
			};

		//! Get TEXTID for name of specified method.
		static TEXTID GetMethodTextID (
			METHOD method
			);

		//! Default constructor.
		CONTRASTPARM ();

		//! Copy constructor.
		CONTRASTPARM (
			const CONTRASTPARM& rhs
			);

		//! Destructor.
		~CONTRASTPARM ();

		//! Assignment.
		CONTRASTPARM& operator= (
			const CONTRASTPARM& rhs
			);

		//! Attach table with bin range and interval.
		//! Ownership of table is transferred to class instance, avoiding need to copy.
		void AttachTable (
			UINT8 *& Table,							//!< Table to attach, will be returned as NULL
			UINT32 NumItems,							//!< Number of items
			double TableBinMinimum,					//!< Table bin minimum (maximum determined based on other parameters)
			double TableBinInterval					//!< Table bin interval
			);

		//! Determine if can recompute table without statistics.
		bool CanComputeWithoutStats () const;

		//! Clear all values.
		void Clear ();

		//! Clear current table without changing any other settings.
		void ClearTable ();

		//! Compute contrast table from statistics and optional specification.
		//! Suitable for all methods.
		ERRVALUE ComputeTable (
			const STAT_UNIVARIATE& stats,		//!< Statistics to compute from
			const float *outspec = 0			//!< Specification array, must have 256 entries
			);

		//! Compute contrast table from current parameters only.
		//! Requires table bin range and interval to be set for some methods.
		//! Not suitable for methods requiring statistics (Normalize, Equalize)
		ERRVALUE ComputeTable ();

		//! Detach table.
		void DetachTable (
			SIMPLE_ARRAY<UINT8>& Table
			) { Table.Swap(m_Table); m_Table.Clear(); }

		//! Get associated image component.
		//! Component returned, no assumptions about value should be made.
		UINT8 GetImageComponent () const
			{ return (m_ImageComponent); }

		//! Get input percentage range.
		const DOUBLE_RANGE& GetInputRangePct (
			) const { return (m_InputRangePct); }

		//! Get input absolute range.
		const DOUBLE_RANGE& GetInputStretchRange (
			) const { return (m_InputRangeAbsolute); }

		//! Get setting for whether inverted.
		bool GetInvert (
			) const { return (m_Invert); }

		//! Get method.
		METHOD GetMethod (
			) const { return (m_Method); }

		//! Get TEXTID for name of current method.
		TEXTID GetMethodTextID () const
			{ return (GetMethodTextID(m_Method)); }

		//! Get "power" value for exponential method.
		double GetPower (
			) const { return (m_Power); }

		//! Get output value to use above maximum bin.
		UINT8 GetOutputAboveMaxBin (
			) const { return (m_OutAboveMaxBin); }

		//! Get output value to use below minimum bin.
		UINT8 GetOutputBelowMinBin (
			) const { return (m_OutBelowMinBin); }

		//! Get output value to use for "constant" method.
		UINT8 GetOutputConstant (
			) const { return (m_OutConstant); }

		//! Get output value to use for invalid input values.
		UINT8 GetOutputInvalid (
			) const { return (m_OutInvalid); }

		//! Get output "median" value for normalize method.
		UINT8 GetOutputMedian (
			) const { return (m_OutMedian); }

		//! Get output "std deviation above median" value for normalize method.
		UINT8 GetOutputStdDevAboveMedian (
			) const { return (m_OutStdDevAboveMedian); }

		//! Get output "std deviation below median" value for normalize method.
		UINT8 GetOutputStdDevBelowMedian (
			) const { return (m_OutStdDevBelowMedian); }

		//! Get output value at maximum input stretch value.
		UINT8 GetOutputStretchMax (
			) const { return (m_OutStretchMax); }

		//! Get output value at minimum input stretch value.
		UINT8 GetOutputStretchMin (
			) const { return (m_OutStretchMin); }

		//! Get table.
		const SIMPLE_ARRAY<UINT8>& GetTable (
			) const { return (m_Table); }

		//! Get table bin interval.
		double GetTableBinInterval (
			) const { return (m_TableBinInterval); }

		//! Get table bin range.
		const DOUBLE_RANGE& GetTableBinRange (
			) const { return (m_TableBinRange); }

		//! Determine if has lookup table.
		bool HasTable (
			) const { return (!m_Table.IsEmpty()); }

		//! Determine if input range specified as percentage.
		bool IsInputRangePct (
			) const { return (m_InputRangeUsePct); }

		//! Process UINT8 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const UINT8 *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process INT8 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const INT8 *ibuffer,						//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process UINT16 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const UINT16 *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process INT16 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const INT16 *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process UINT32 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const UINT32 *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process INT32 values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const INT32 *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process "float" values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const float *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process "double" values.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const double *ibuffer,					//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer								//!< Output buffer
			) const;

		//! Process values with specified NUMTYPE.
		ERRVALUE Process (
			UINT32 count,								//!< Number of values
			const void *ibuffer,						//!< Input buffer
			const UINT8 *validity,					//!< Validity buffer, NULL if all values valid
			UINT8 *obuffer,							//!< Output buffer
			NUMTYPE NumType							//!< Numeric type
			) const;

		//! Set to "constant" method with value specified.
		void SetConstant (
			UINT8 OutConstant
			);

		//! Set to "exponential" method with power specified.
		void SetExponential (
			double power						//!< Power value > 0
			);

		//! Set to "hyperbolic tangent" method with scale specified.
		void CONTRASTPARM::SetHyperbolicTangent (
			double scale						//!< Scale value > 0, typically < 10
			);

		//! Set associated image component.
		void SetImageComponent (
			UINT8 ImageComponent				//!< Image component
			) { m_ImageComponent = ImageComponent; }

		//! Set input value stretch range.
		//! Automatically sets to use percentage when computing table.
		void SetInputRangePct (
			double MinPct,						//!< Minimum percentage (0-100)
			double MaxPct						//!< Maximum percentage (0-100)
			);

		//! Set whether to use input percentage range or absolute range.
		void SetInputRangeUsePct (
			bool InputRangeUsePct					//!< True if range is percentage, false if absolute
			) { m_InputRangeUsePct = InputRangeUsePct; }

		//! Set input value stretch maximum.
		void SetInputStretchMax (
			double value
			) { m_InputRangeAbsolute.SetMaximum(value); }

		//! Set input value stretch minimum.
		void SetInputStretchMin (
			double value
			) { m_InputRangeAbsolute.SetMinimum(value); }

		//! Set absolute input value stretch range.
		//! Automatically sets to use absolute range when computing table.
		void SetInputStretchRange (
			const DOUBLE_RANGE InputRangeAbsolute
			) { m_InputRangeUsePct = false; m_InputRangeAbsolute = InputRangeAbsolute; }

		//! Set whether inverted bright/dark.
		void SetInvert (
			bool Invert
			) { m_Invert = Invert; }

		//! Set contrast method.
		void SetMethod (
			METHOD method,							//!< Method to set
			bool usedefaults = true				//!< Set any value-specific parameters to defaults
			);

		//! Set to "normalize" method with parameters.
		void SetNormalize (
			UINT8 median,
			UINT8 StDevBelowMedian = 50,
			UINT8 StDevAboveMedian = 50
			);

		//! Set value to output above maximum bin.
		void SetOutputAboveMaxBin (
			UINT8 OutAboveMaxBin
			) { m_OutAboveMaxBin = OutAboveMaxBin; }

		//! Set value to output below minimum bin.
		void SetOutputBelowMinBin (
			UINT8 OutBelowMinBin
			) { m_OutBelowMinBin = OutBelowMinBin; }

		//! Set value to output for "invalid" input.
		void SetOutputInvalid (
			UINT8 OutInvalid
			) { m_OutInvalid = OutInvalid; }

		//! Set output median value for normalize method.
		//! Does not change method itself.
		void SetOutputMedian (
			UINT8 median
			) { m_OutMedian = median; }

		//! Set output standard deviation above median for normalize method.
		//! Does not change method itself.
		void SetOutputStdDevAboveMedian (
			UINT8 StdDevAboveMedian
			) { m_OutStdDevAboveMedian = StdDevAboveMedian; }

		//! Set output standard deviation below median for normalize method.
		//! Does not change method itself.
		void SetOutputStdDevBelowMedian (
			UINT8 StdDevBelowMedian
			) { m_OutStdDevBelowMedian = StdDevBelowMedian; }

		//! Set value to output at maximum input stretch value.
		void SetOutputStretchMax (
			UINT8 OutStretchMax
			) { m_OutStretchMax = OutStretchMax; }

		//! Set value to output at minimum input stretch value.
		void SetOutputStretchMin (
			UINT8 OutStretchMin
			) { m_OutStretchMin = OutStretchMin; }

		//! Set table with bin range and interval
		ERRVALUE SetTable (
			const SIMPLE_ARRAY<UINT8>& Table,	//!< Table to set
			double TableBinMinimum,					//!< Table bin minimum (maximum determined based on number of items in table)
			double TableBinInterval					//!< Table bin interval
			);

		//! Set table bin range and interval.
		ERRVALUE SetTableBinRange (
			const DOUBLE_RANGE TableBinRange,
			double BinInterval
			);

		//! Get item definition array for serialization.
		static const SERIALIZERITEM* SerialGetItemDef ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef CONTRASTPARM THISCLASS;

		DOUBLE_RANGE m_InputRangePct;			// Input stretch percentage range
		DOUBLE_RANGE m_InputRangeAbsolute;	// Input stretch absolute range
		DOUBLE_RANGE m_TableBinRange;			// Table bin range
		double m_TableBinInterval;				// Table bin interval
		double m_Power;							// Power for exponential methd
		METHOD m_Method;							// Contrast method
		UINT8 m_OutStretchMin;					// Output value for minimum stretch value
		UINT8 m_OutStretchMax;					// Output value for maximum stretch value
		UINT8 m_OutMedian;						// Median for normalization method
		UINT8 m_OutStdDevBelowMedian;			// Std deviation below median for normalization
		UINT8 m_OutStdDevAboveMedian;			// Std deviation above median for normalization
		UINT8 m_OutBelowMinBin;					// Value to output if input < bin range minimum
		UINT8 m_OutAboveMaxBin;					// Value to output if input >= bin range maximum
		UINT8 m_OutInvalid;						// Value to output if input not valid
		UINT8 m_OutConstant;						// Value to output if IsConstant bit set
		UINT8 m_ImageComponent;					// Image component
		bool m_Invert;								// Invert table
		bool m_UseDirectUINT8;
		bool m_InputRangeUsePct;				// Input range is percentage
		SIMPLE_ARRAY<UINT8> m_Table;			// Lookup table

		ERRVALUE ComputeBinIdxRangeUsed (int& MinBinIdx, int& MaxBinIdx) const;
		void ComputeNormalSpec (float *spec) const;
		ERRVALUE ComputeTableNoStats (int NumBins = 0);
		void GetBinMinMaxInt (INT32& BinMinVal, INT32& BinMaxVal, INT32& BinInterval) const;
		void GetBinMinMaxInt (UINT32& BinMinVal, UINT32& BinMaxVal, UINT32& BinInterval) const;
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};


#undef	LIBEXPORT
#undef	CLASSEXPORT

#endif	// INC_CONTRASTPARM_H

