/**************************************************************************
 *
 *	\file cpuext.h <mi32/cpuext.h>
 *	\brief CPU Extension support routines
 *
 *		Note. This file can be safely included for any processor
 *
 * \if NODOC
 * $Log: cpuext.h_v $
 * Revision 1.20  2008/10/14 13:12:17  mju
 * Doxygen fix.
 *
 * Revision 1.19  2006/03/10 17:00:55  mju
 * Disallow win64.
 *
 * Revision 1.18  2006/02/16 16:13:55  dwilliss
 * changed an ifdef to use new define instead of deprecated one
 *
 * Revision 1.17  2005/07/15 14:57:19  mju
 * Add comment as to why string hjdr gets included.
 *
 * Revision 1.16  2005/07/15 14:01:47  mju
 * Include memory and string hdrs so works correctly again.
 *
 * Revision 1.15  2005/07/14 17:35:55  mju
 * Remove stdansic hdr.
 *
 * Revision 1.14  2004/07/06 22:11:54  dwilliss
 * *** empty log message ***
 *
 * Revision 1.13  2004/06/29 22:03:24  dwilliss
 * Added more CPU feature flags
 *
 * Revision 1.12  2004/06/15 14:43:30  dwilliss
 * Defined readTSC() (Taken from Dr Dobbs Journal, May 2004, page 23)
 *
 * Revision 1.11  2004/01/30 22:23:59  dwilliss
 * Only try to include wchar.h on WIN32. Not necessary on Unix and doesn't
 * exist anyway.
 *
 * Revision 1.10  2003/12/29 22:04:10  scowan
 * Include wchar so that we do not corrupt memcpy for it.
 *
 * Revision 1.9  2003/12/12 20:28:00  dwilliss
 * Added protos for some new alpha blending funcitons
 *
 * Revision 1.8  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.7  2002/02/26 22:56:33  dwilliss
 * More prototypes
 *
 * Revision 1.6  2002/02/20 23:23:03  dwilliss
 * Added 32-bit alpha blending
 *
 * Revision 1.5  2001/05/18 20:40:35  dwilliss
 * Added more prototypes and documentation
 *
 * Revision 1.4  2001/05/17 14:40:22  dwilliss
 * Added PREFETCHNTA flag
 *
 * Revision 1.3  2001/05/16 21:15:29  dwilliss
 * Move #define constants so they they're always defined
 *
 * Revision 1.2  2001/05/16 20:38:59  dwilliss
 * Make amemcpy() match memcpy() in signature
 *
 * Revision 1.1  2001/05/16 19:44:49  dwilliss
 * Initial revision
 *
 *
 * \endif
 */
#ifndef INC_MI32_CPUEXT_H
#define INC_MI32_CPUEXT_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_STDDEF_H
#include <stddef.h>	// For size_t define
#define	INC_STDDEF_H
#endif

// string.h contains the original memcpy prototype.
// In stddefns.h we may have defined memcpy as amemcpy, so the prototype
// from string.h must be defined first to avoid compiler errors.
#ifndef	INC_STRING_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <string.h>
#define	INC_STRING_H
#endif
#endif

#ifdef WIN32
#ifndef _INC_WCHAR
#include <wchar.h>			// To not override memcpy() define in wchar.h
#endif
#endif


#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

#define	CPUEXT_MMX				0x00000001	//!< Basic MMX support
#define	CPUEXT_SIMD				0x00000002	//!< Old name for SSE
#define	CPUEXT_SSE				0x00000002	//!< Old name for SSE
#define	CPUEXT_PREFETCHNTA	0x00000004	//!< Intel cashability instructions
#define	CPUEXT_SSE2				0x00000008	//!< 
#define	CPUEXT_CMOV				0x00000010	//!< 
#define	CPUEXT_TSC				0x00000020	//!< 
#define	CPUEXT_3DNOW			0x00010000	//!< AMD 3DNow
#define	CPUEXT_3DNOWEX			0x00020000	//!< AMD 3DNow extensions
#define	CPUEXT_AMDMMX			0x00040000	//!< AMD's MMX extensions


#if defined(__cplusplus)
extern "C" {
#endif

#if defined(LINUX) && defined(INTEL)
	// Although we never use it, this version of TSC is supposed to work on
	// gcc.  I got this version from a letter in Dr. Dobbs Journal, July 2004
	// Page 8.  It's here to preserve it for postarity and so that I won't have
	// to figure it out again later.
	#define readTSC() ({unsigned long long __scr; __asm__ __volatile__ ("rdtsc" :"=A"(__scr));__scr;})
#endif

#if defined(WIN32) && !defined(WIN64)

	//! Check to see what CPU extensions are supported by the processor
	//!
	//!	Checks to see if the machine supports various acceleration
	//! technologies.  Returns flags indicating what it has.
	//!
	//! Possible return values:
	//!		CPUEXT_MMX				Basic MMX support
	//!		CPUEXT_SIMD				Streaming SIMD support
	//!		CPUEXT_PREFETCHNTA	Intel cashability instructions
	//!
	//! Others may be added in the future.
	//!
	LIBEXPORT UINT32 GetCPUExtension();

	//! Read the processor Time Stamp Counter (Windows only)
	//! NOTE! this function will only work on newer Intel processors and 
	//! should only be used in testing code, not in production, as it will 
	//! crash on older Pentiums.  It returns the # of CPU clock cycles since
	//! the last reset.
	inline __int64 readTSC() { __asm { rdtsc } }

#ifndef GENERATING_DOXYGEN_OUTPUT
	//! These are pointers to functions which can be used like normal
	//! functions.  They are statically initialized so that the first
	//! time they are called they check to see if MMX is available or
	//! not and change the pointer to either an MMX-accelerated or 
	//! normal version, then call that.  Later calls just end up 
	//! calling the right version for the current machine.

	//! NOTE:  Since the point here is speed, this is NOT in misystem.
	//! The overhead of calling into a DLL is worse than a normal
	//! function call.
	//!  If you use these in a DLL, just suck the source module in.
	
	//! Accelerated memcpy.
	//! Like memcpy() but MMX-accelerated.  On platforms where there's
	//! no chance of MMX, this is just a #define that calls memcpy()
	//! Implemented in amemcpy.c
	extern void* (*amemcpy)(void* dest, const void* src, size_t size);
#endif //!< GENERATING_DOXYGEN_OUTPUT

#else		//!< Not a chance!

#define GetCPUExtension() (0)
#define amemcpy memcpy

#endif	//!< Intel

#define HasMMX() (GetCPUExtension() & CPUEXT_MMX)

#if 0	// Fake function prototypes for Genitor to generate documentation from

//! Accelerated memcpy.
//! Like memcpy() but MMX-accelerated.  On platforms where there's
//! no chance of MMX, this is just a #define that calls memcpy()
void* amemcpy (
	void* dest, 
	const void* src, 
	size_t size
	);

#endif		//!< End of fake functions

//! Do alpha blending of a source buffer into a dest buffer.
//! This version is for 24-bit source onto 24-bit dest.
//! n is number of pixels.
//! alpha is in the range 0-255 (although the extremes are
//! rather pointless, as 0 would leave the dest untouched and
//! 255 is an expensive way to do a memcpy)
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlend24 (
	UINT8* dest, 
	const UINT8* src, 
	UINT8 alpha, 
	int n
	);

//! Do alpha blending of a source buffer into a dest buffer.
//! This version is for 24-bit source onto 24-bit dest.
//! n is number of pixels.
//! alpha is in the range 0-255 
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlendVarAlpha24 (
	UINT8* dest, 
	const UINT8* src, 
	const UINT8* alpha, 		//!< Alphas for each pixel. 
	int n
	);

//! Do alpha blending of a solid color onto a destination buffer.
//! The color is broken up into 3 componants because the places where this
//! will be the most useful, we already have them like this.
//! implemented in aalphab.c
//! Dest is assumed to be 24-bit RGB data.  If it's really BGR, pass your
//! color componants backwards.
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlendColor24 (
	UINT8* dest, 
	UINT8 r, 
	UINT8 g,
	UINT8 b, 
	UINT8 alpha, 
	int n
	);

//! Do alpha blending of a source buffer into a dest buffer.
//! This version is for 32-bit source onto 32-bit dest.
//! n is number of pixels.
//! alpha is in the range 0-255 (although the extremes are
//! rather pointless, as 0 would leave the dest untouched and
//! 255 is an expensive way to do a memcpy)
//!
//! This version assumes a constant alpha.
//! Note that all 4 color componants are blended because it's faster
//! to do that (4 bytes at a time) than doing 3 individual componants
//! and skipping the 4th.  It also means this function doesn't have to
//! care what the RGB order is and which byte to skip.
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlend32 (
	UINT32* dest, 
	const UINT32* src, 
	UINT8 alpha, 
	int n
	);

//! Do alpha blending of a source buffer into a dest buffer.
//! This version is for 32-bit source onto 32-bit dest.
//! n is number of pixels.
//! alpha is in the range 0-255 
//!
//! Note that all 4 color componants are blended because it's faster
//! to do that (4 bytes at a time) than doing 3 individual componants
//! and skipping the 4th.  It also means this function doesn't have to
//! care what the RGB order is and which byte to skip.
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlendVarAlpha32 (
	UINT32* dest, 
	const UINT32* src, 
	const UINT8* alpha, 
	int n
	);

//! Do alpha blending of a solid color onto a destination buffer.
//! implemented in aalphab.c
//! 
//! Note that all 4 color componants are blended because it's faster
//! to do that (4 bytes at a time) than doing 3 individual componants
//! and skipping the 4th.  It also means this function doesn't have to
//! care what the RGB order is and which byte to skip.
//!
//! On platforms where it's possible, this function checks for the
//! availability of MMX and uses it if it's there
void AlphaBlendColor32 (
	UINT32* dest, 
	UINT32 pix, 
	UINT8 alpha, 
	int n
	);


//! Do alpha blending of a solid color onto a destination buffer.
//! implemented in aalphab.c
//! 
//! This is similar to AlphaBlendColor24() but the dest buffer is assumed
//! to be made up of 32-bit pixels in which the lowest 8 bits are "alpha".
//! Note that this is not based on the order of the bytes in the buffer,
//! but on bits in a 32-bit integer.  It will work for 0xRRGGBBAA or
//! 0xBBGGRRAA.
//! Also note that for this one, the "alpha" value is part of the pixel
//! passed in.
void AlphaBlendColor32RGBA (
	UINT32* dest, 
	UINT32 pix, 
	int n
	);


//! Do alpha blending of a solid color onto a destination buffer.
//! implemented in aalphab.c
//! 
//! This is similar to AlphaBlendColor24() but the dest buffer is assumed
//! to be made up of 32-bit pixels in which the highet 8 bits are "alpha".
//! Note that this is not based on the order of the bytes in the buffer,
//! but on bits in a 32-bit integer.  It will work for 0xAABBGGRR or
//! 0xAARRGGBB.
//! Also note that for this one, the "alpha" value is part of the pixel
//! passed in.
void AlphaBlendColor32ABGR (
	UINT32* dest, 
	UINT32 pix, 
	int n
	);


#if defined(__cplusplus)
}
#endif

#endif
