/**
*	credentialscache.h
*
*	$Id: credentialscache.h_v 1.2 2012/11/30 19:10:29 bcolombe Exp $
*
*	$Log: credentialscache.h_v $
*	Revision 1.2  2012/11/30 19:10:29  bcolombe
*	Add GetUserList
*
*	Revision 1.1  2012/09/10 19:40:53  bcolombe
*	Initial revision
*
*
**/

#ifndef INC_MI32P_CREDENTIALSCACHE_H
#define INC_MI32P_CREDENTIALSCACHE_H

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
	#include <mi32/critsec.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class XMLNODE;

class CLASSLIBEXPORT CREDENTIALSCACHE {
	public:	

		//! Delegate to prompt for userid and password.
		//! Delegate should validate credentials when OK pressed before closing.
		//! SaveCredentials should be called after validation.
		//! If delegate cannot validate, then credentials should not be saved
		typedef fastdelegate::FastDelegate<ERRVALUE (const MISTRING& UsageString, MISTRING& UserID, MISTRING& Password)> DELEGATE_PROMPT;
			
		enum GETFLAGS {
			GETFLAG_None,
			GETFLAG_PromptAlways
			};
	 
		enum SAVEMODE {
			SAVEMODE_FileUserID,		// Retain only userid in file
			SAVEMODE_FileUserPassword,	// Retain both username and password in file
			SAVEMODE_SessionOnly		// Retain credentials for session only
			}; 

		//! Get credentials with prompt.
		//! If not found or GETFLAG_PromptAlways, will prompt.  Prior to any prompt, existing credentials will be removed,
		//! thus if prompt is cancelled the user will be required to provide credentials on future attempts.
		//! Will return EUserCancel if user cancels prompt.
		static ERRVALUE GetCredentials (
			const MISTRING& UsageString,			//!< Database connection string, URL, etc.
			MISTRING& UserID,						//!< If blank, get last used UserID for UsageString.  If not found, will prompt.
			MISTRING& Password,						//!< If not found, will prompt
			DELEGATE_PROMPT delegate,				//!< Delegate to prompt user for credentials
			GETFLAGS flag = GETFLAG_None			//!< If GETFLAG_PromptAlways, always show prompt, even if credentials found.
			);
	
		//! Get list of users for given usage string
		static ERRVALUE GetUserList (
			const MISTRING& UsageString,			//!< Database connection string, URL, etc.
			MISTRINGLIST& UserIDs					//!< List of users
			);

		//! Remove all credentials for specified UsageString.
		static ERRVALUE RemoveCredentials (
			const MISTRING& UsageString
			);
	 
		//! Remove credentials for specified UsageString/UserID combination.
		static ERRVALUE RemoveCredentials (
			const MISTRING& UsageString,
			const MISTRING& UserID
			);

		//! Save credentials.
		static ERRVALUE SaveCredentials (
			const MISTRING& UsageString,			//!< Database connection string, URL, etc.
			const MISTRING& UserID,
			const MISTRING& Password,
			SAVEMODE SaveMode = SAVEMODE_SessionOnly,
			UINT32 ExpireSeconds = 0				//!< Expiration time in seconds, 0 to expire at process exit (unless save to file)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CREDENTIALSCACHE ();
		~CREDENTIALSCACHE ();

		static CREDENTIALSCACHE& GetCredentialsCache (
			);

		ERRVALUE GetCredentialsPriv (
			const MISTRING& UsageString,
			MISTRING& UserID,
			MISTRING& Password,
			DELEGATE_PROMPT delegate,
			GETFLAGS flag
			);

		ERRVALUE GetUserListPriv (
			const MISTRING& UsageString,
			MISTRINGLIST& UserIDs
			);
	
		ERRVALUE RemoveCredentialsPriv (
			const MISTRING& UsageString
			);
	 
		ERRVALUE RemoveCredentialsPriv (
			const MISTRING& UsageString,
			const MISTRING& UserID
			);

		ERRVALUE SaveCredentialsPriv (
			const MISTRING& UsageString,
			const MISTRING& UserID,
			const MISTRING& Password,
			SAVEMODE SaveMode,
			UINT32 ExpireSeconds
			);

		struct ITEM {
			MISTRING m_UsageString;
			MISTRING m_UserID;
			MISTRING m_Password;
			UINT32 m_TimeToExpire;
			UINT32 m_ExpireSeconds;
			
			ITEM () : m_TimeToExpire(0), m_ExpireSeconds(0) {}
			};

		MITHREAD::CRITICALSECTION m_CredentialsCacheCS;
		MILIST<ITEM> m_CredentialsCache;

		bool IsConnectMatch (const MISTRING& LHSUsageString, const MISTRING& RHSUsageString) const;
		bool IsConnectMatch (const MISTRING& LHSUsageString, const MISTRING& LHSUserId, const MISTRING& RHSUsageString, const MISTRING& RHSUserID) const;

		CREDENTIALSCACHE (const CREDENTIALSCACHE& rhs);
		CREDENTIALSCACHE& operator=(const CREDENTIALSCACHE& rhs);
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
#undef	CLASSLIBEXPORT
#endif		// INC_MI32_CREDENTIALSCACHE_H
