/******************************************************************************
 *
 *	\file mi32/httpserver.h
 *	\brief HTTP Server
 *
 * \if NODOC
 * $Log: httpserver.h_v $
 * Revision 1.5  2005/07/15 16:33:22  mju
 * Fix ending endif.
 *
 * Revision 1.4  2005/04/12 14:48:34  dwilliss
 * Renamed our class to avoid name conflict with Windows includes
 * Made destructor virtual
 *
 * Revision 1.3  2004/01/29 18:19:40  dwilliss
 * *** empty log message ***
 *
 * Revision 1.2  2004/01/08 16:41:43  dwilliss
 * More complete now
 *
 * Revision 1.1  2004/01/02 22:58:03  dwilliss
 * Initial revision
 *
 * \endif
 *
 *****************************************************************************/

#ifndef INC_MI32_HTTPSERVER_H
#define INC_MI32_HTTPSERVER_H

#ifndef INC_MI32_HTTPCLIENT_H
#include <mi32/httpclient.h>
#endif

#ifndef INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


class CLASSLIBEXPORT HTTPSERVER {
	public:
	
		class PATH;
		class HANDLER;
		class FILEHANDLER;
		class UPLOADHANDLER;
		class CLIENT;

		HTTPSERVER();
		virtual ~HTTPSERVER();

		//! Add a listening socket to the server.
		//! This version will take care of creating and binding the socket to
		//! the address you provide.
		ERRVALUE AddListenSocket (
			const MISOCKET::ADDR& address
			);

		//! Add a listening socket to the server.
		//! The socket should already be bound to a port.
		//! \param sock The socket to add
		//! \param bTakeOwnership If true (the default), the server will "own" 
		//! 	the socket and delete it when it shuts down.  If false, you must
		//! 	delete it.
		ERRVALUE AddListenSocket (
			MISOCKET* sock,
			bool bTakeOwnership = true
			);

		//! Augment the server's list of known MIME types.
		//! There are a few types built-in (htm, html, gif, jpg, png, etc)
		ERRVALUE AddMIMEType (
			const char* extn,			//!< File extension to associate (no dot)
			const char* mimetype		//!< MIME type (eg: "text/html")
			);

		//! Add a file type handler
		//! The handler is then "owned" by the server and will be deleted by the
		//! server's destructor.  
		ERRVALUE AddHandler (
			HANDLER* handler
			);

		//! Add a new path.  
		//! The path is then "owned" by the server and will be deleted by the
		//! server's destructor.
		ERRVALUE AddPath (
			PATH* path
			);

		//! Generate an HTML status page giving the server status
		void GenerateStatusPage (
			MISTRING& mistr
			);


		// Find the MIME type for a given file.
		// \return a MIME type string if know, NULL if not
		const char* FindMIMEType (
			const FILEPATH& filepath
			);

		//! Find the virtual path that matches a given message.
		//! Returns NULL if no paths match.
		PATH* FindPath (
			const HTTPMESSAGE& msg
			);

		//! Close all client and listening sockets and shutdown.
		//! If running our own MainLoop() exit from that too.
		void Shutdown (
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		struct MIMEEXTN {
			char* extn;
			char* mime;
			};

		class LISTENER;
		friend class LISTENER;	// Needs access to RemoveListener()
		friend class CLIENT;		// Needs access to RemoveClient()
		friend class PATH;		// Needs access to handler list

		// Add a new client.  Only called by a listener
		void AddClient (
			CLIENT* client
			);

		// Remove a client.  Only called by clients in their OnClose methods
		void RemoveClient (
			CLIENT* client
			);

		// Remove a listener.  Only called by listeners in their distructors
		void RemoveListener (
			LISTENER* listener
			);
#endif

		//! Should allocate a new client.
		//! The default implementation allocates a new HTTPSERVER::CLIENT
		virtual HTTPSERVER::CLIENT* v_AllocateClient (
			MISOCKET* sock
			);

		//! Will be called when a new client is added
		virtual void v_OnClientConnect (
			CLIENT* client
			);

		//! Will be called when a client disconnects.
		//! unless the disconnect comes from the server's destructor
		virtual void v_OnClientDisconnect (
			CLIENT* client
			);
#ifndef GENERATING_DOXYGEN_OUTPUT

		// copy const and assignment operator private and unimplemented.
		HTTPSERVER(const HTTPSERVER&);
		HTTPSERVER& operator=(const HTTPSERVER&);

		static const MIMEEXTN s_BuiltInMIMETypes[];
		SIMPLE_ARRAY<LISTENER*> m_Listeners;
		SIMPLE_ARRAY<CLIENT*> m_Clients;
		SIMPLE_ARRAY<PATH*> m_Paths;
		SIMPLE_ARRAY<HANDLER*> m_Handlers;
		SIMPLE_ARRAY<MIMEEXTN> m_MIMETypes;
		bool m_bShuttingDown;

#endif

	};


/****************************************************************************/

//! Class to represent a virtual path in the HTTP server
//! The constructor takes a virtual path. Any request which falls within
//! this path will be processed by calling ProcessRequest
//!
class CLASSLIBEXPORT HTTPSERVER::PATH {
	public:
		PATH (
			HTTPSERVER& server,
			const char* virtualpath
			);

		virtual ~PATH();

		//! Add a handler to the path.
		//! If a path doesn't implement ProcessRequest, the default implementation
		//! will find the best handler and use that.  If no handlers are added
		//! to a path, it will use the server's handler list.  
		ERRVALUE AddHandler (
			HTTPSERVER::HANDLER* handler
			);

		//! Returns true if our VirtualPath matches the first part of the
		//! path passed in.
		bool Matches (
			const char* path
			) const;

		HTTPSERVER& GetServer (
			);

		const char* GetVirtualPath (
			) const;

		//! Process a request. 
		//! Returns true if a reply was sent back to the client.
		bool ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			);

		//! Set wether or not a method is allowed in this directory.
		//! By default, the only methods allowed are GET, HEAD, and POST
		void SetMethodAllowed (
			HTTPMESSAGE::METHOD method,
			bool bIsAllowed = true
			);

	private:

		HTTPSERVER& m_Server;
		STRUTF8 m_VirtualPath;	// Path relative to server root
		SIMPLE_ARRAY<HTTPSERVER::HANDLER*> m_Handlers;
		bool m_AllowedMethods[HTTPMESSAGE::METHOD_NUM_METHODS];	// 

		//! Process a request. 
		//! Should return true if a reply was sent back to the client.
		//! If false is returned, the client will be sent a 404 Not Found error.
		virtual bool v_ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			);

	};

/****************************************************************************/
//! Class to represent a virtual path in the HTTP server
//! The constructor takes a virtual path. Any request which falls within
//! this path will be processed by calling ProcessRequest
//!
class CLASSLIBEXPORT HTTPSERVER::HANDLER {
	public:
		HANDLER (
			HTTPSERVER& server,
			const char* name		//!< Used for config file settings
			);

		virtual ~HANDLER();

		// Get name used for configuration file settings
		const char* GetName (
			);

		HTTPSERVER& GetServer (
			);

		//! Determine how well this handler could service the request.
		//! \return 0 to indicate not at all, 100 for complete match,
		//! anything inbetween for partial match.
		//! This allows you to define a handler of last resort.
		int Matches (
			const HTTPMESSAGE& request
			) const;

		//! Process a request. 
		//! Should return true if it sent a reply back to the client
		//! If false is returned, the client will be sent a 404 Not Found
		//! error.
		bool ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			);

		// Determine if this handler can handle a specific method
		bool CanHandleMethod (
			HTTPMESSAGE::METHOD method
			) const;

	protected:

	private:

		// Should return if this handler can handle a specific method.
		// Returns false otherwise.
		// This does not need to worry about permissions set on the virtual
		// path - that's handled by the caller.
		virtual bool v_CanHandleMethod (
			HTTPMESSAGE::METHOD method
			)  const = 0;

		//! Determine how well this handler could service the request.
		//! \return 0 to indicate not at all, 100 for complete match,
		//! anything inbetween for partial match.
		//! This allows you to define a handler of last resort.
		//! This function shouldn't try to determine if the file actually
		//! exists or is usable, just that it's the right type to try.
		virtual int v_Matches (
			const HTTPMESSAGE& request
			) const = 0;

		//! Process a request. 
		//! Should return true if it sent a reply back to the client
		//! If false is returned, the client will be sent a 404 Not Found
		//! error.
		virtual bool v_ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			) = 0;

		HTTPSERVER& m_Server;
		const char* m_name;

	};

/****************************************************************************/
//! Class to represent a virtual path in the HTTP server from which the user
//! can GET files.
//! This handler does the following security checks...
//!	1. The request URI may not have any .. path componants in it.
//!	2. If the URI is for a folder, it will look for and return to the
//!		the user an index.htm or index.html (in that order)
//!	3. Any files to be retrieved must be readable by the server
//!
class CLASSLIBEXPORT HTTPSERVER::FILEHANDLER : public HTTPSERVER::HANDLER {
	public:

		FILEHANDLER (
			HTTPSERVER& server,
			const FILEPATH& DocumentRoot
			);

		virtual ~FILEHANDLER();

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual int v_Matches (
			const HTTPMESSAGE& request
			) const;

		virtual bool v_CanHandleMethod (
			HTTPMESSAGE::METHOD method
			) const;

		virtual bool v_ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			);

		FILEPATH m_DocumentRoot;
#endif
	};

/****************************************************************************/
//! Class to represent a virtual path in the HTTP server for uploads via PUT.
//! Warning! This class is stupid and doesn't do any permission checking other
//! than the following...
//!	1. The destination directory must already exist and be writable by
//!		the server proccess.
//!	2. If the file to be written already exists, it must be writable.
//!	3. The request is not allowed to have any .. in its path to prevent
//!		spoofing it to write where it's not supposed to.
//!
class CLASSLIBEXPORT HTTPSERVER::UPLOADHANDLER : public HTTPSERVER::HANDLER {
	public:

		UPLOADHANDLER (
			HTTPSERVER& server,
			const FILEPATH& DocumentRoot
			);

		virtual ~UPLOADHANDLER();

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual int v_Matches (
			const HTTPMESSAGE& request
			) const;

		virtual bool v_CanHandleMethod (
			HTTPMESSAGE::METHOD method
			) const;

		virtual bool v_ProcessRequest (
			HTTPSERVER::CLIENT* client,
			const HTTPMESSAGE& request
			);

		FILEPATH m_DocumentRoot;
#endif
	};


/****************************************************************************/

//! Server side representation of a client.
//! If you want to derive your own client, override the server's 
//! v_AllocateClient() method and have allocate your derive class via new
class CLASSLIBEXPORT HTTPSERVER::CLIENT : public HTTPCLIENT {
	public:
		CLIENT (
			HTTPSERVER& server,
			MISOCKET* sock
			);

		~CLIENT();

		// Send an error message back to the client.
		// If err is < 0, the MicroImages error message will be included
		void SendErrorReply (
			HTTPMESSAGE::STATUS status,
			const HTTPMESSAGE& request,
			ERRVALUE err = 0
			);


	private:

		// virtual, Implemented from MISOCKET::CLIENT, but derived class cannot
		// override it
		void v_OnClose (
			);

		// virtual, Implemented from MISOCKET::CLIENT, but derived class cannot
		// override it
		void v_OnMessage (
			const HTTPMESSAGE&
			);

		HTTPSERVER& m_Server;
		bool m_bDestroying;
	};


#undef CLASSLIBEXPORT


#endif // INC_MI32_HTTPSERVER_H
