/**
 * \file mi32/mistatus.h
 * \brief MISTATUS class definitions.
 *
 * \if NODOC
 * $Id: mistatus.h_v 1.22 2012/09/18 13:48:29 mju Exp $
 *
 * $Log: mistatus.h_v $
 * Revision 1.22  2012/09/18 13:48:29  mju
 * Dox ctor fix.
 *
 * Revision 1.21  2012/09/04 15:27:09  mju
 * Add statusmanager::saverestore.
 *
 * Revision 1.20  2011/11/02 14:29:35  mju
 * Remove redundant hdrs.
 *
 * Revision 1.19  2009/03/23 13:25:25  mju
 * Add v_reportBegin for statuslogger so will log beginning of task.
 *
 * Revision 1.18  2007/12/07 22:08:19  mju
 * In reportTimeElapsed return bool for whether changed.
 *
 * Revision 1.17  2007/12/05 18:54:36  mju
 * In statuscontext.TestCancelled check if allowed to cancel.
 *
 * Revision 1.16  2007/12/05 18:16:46  mju
 * Add reportProgress/Increment/NoCancel.
 *
 * Revision 1.15  2007/11/16 18:21:59  scowan
 * Set isCancelled if detachMgr is called.
 *
 * Revision 1.14  2007/11/12 15:31:30  mju
 * Add context default ctor and create methods.
 *
 * Revision 1.13  2007/11/09 21:16:26  mju
 * Add context.hasManager.
 *
 * Revision 1.12  2007/11/09 14:19:08  mju
 * Change context report... methods to return EUserCancel or 0.
 *
 * Revision 1.11  2007/11/08 16:47:22  mju
 * Add updateUI.
 *
 * Revision 1.10  2007/11/08 14:09:57  mju
 * Add isCancelSet.
 *
 * Revision 1.9  2007/11/08 13:41:49  mju
 * Add resetTimer and m_lastElapsed.
 *
 * Revision 1.8  2007/11/07 22:14:10  mju
 * Add setCancelled.
 *
 * Revision 1.7  2007/11/02 21:44:58  mju
 * Add methods to disable/enable logging.
 * Add overridable for reporting time elapsed.
 * Keep timer in mistatusmanager.
 *
 * Revision 1.6  2007/11/02 15:22:19  mju
 * Add clearContextList.
 * Add context.detachMgr.
 *
 * Revision 1.5  2007/10/19 14:26:53  mju
 * Make getActiveManager public.
 *
 * Revision 1.4  2007/10/18 22:10:12  scowan
 * Fixed inclusion guard misspelling.
 *
 * Revision 1.3  2007/10/18 21:36:05  scowan
 * Added critical section lock to getContextList.
 *
 * Revision 1.2  2007/10/18 20:20:30  mju
 * Add getContextList and typedef for contextlist.
 *
 * Revision 1.1  2007/10/18 18:06:13  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MISTATUS_H
#define	INC_MI32_MISTATUS_H

#ifndef	INC_MI32_MISYSTEM_H
#include <mi32/misystem.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
#include <mi32/critsec.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_MTIMER_H
#include <mi32/mtimer.h>
#endif


#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class MISTATUSCONTEXT;

//===================================================================================================================

//! Base status manager.
//! Most methods should not be called directly as they are called by MISTATUSCONTEXT where needed.
class CLASSLIBEXPORT MISTATUSMANAGER {
	public:

		class SAVERESTORE {
			public:
				SAVERESTORE (
					) : m_pManager(MISTATUSMANAGER::GetActiveManager()) { }
				~SAVERESTORE (
					) { MISTATUSMANAGER::SetActiveManager(m_pManager); }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTATUSMANAGER *m_pManager;
				SAVERESTORE (const SAVERESTORE&);
				SAVERESTORE& operator= (const SAVERESTORE&);
				#endif
			};

		//! Get active status manager for current thread.
		//! Typically only used if creating status context is conditional upon having enough
		//! steps in a task for reporting to be worthwhile.
		static MISTATUSMANAGER* GetActiveManager ();

		//! Set active status manager for current thread.
		//! @return Previous active status manager.
		//! Use care when retaining previous active status manager to restore later,
		//! generally it will only be safe to do so in the main application thread.
		static MISTATUSMANAGER* SetActiveManager (
			MISTATUSMANAGER *manager
			);

		//! Set status manager creator for current thread.
		//! For interactive applications this is usually only called for main application thread.
		//! The creator is used when no active manager is set and a STATUSCONTEXT is constructed.
		static void SetCreator (
			FastDelegate<MISTATUSMANAGER*()> DelegateCreateManager
			);

		//! Default constructor.
		//! Will set newly constructed manager as active manager for current thread.
		MISTATUSMANAGER ();

		//! Destructor.
		//! If this instance is the active manager for the current thread, the active manager
		//! will be set to NULL.
		virtual ~MISTATUSMANAGER ();

		//! Get timer.
		MTIMER& GetTimer (
			) { return (m_Timer); }

		//! Check if operation has been cancelled.
		//! When cancelled state is first detected, all attached contexts will be set as cancelled.
		bool IsCancelled (
			);

		//! Report time elapsed.
		//! @return true if elapsed time different than last report, false if not
		bool ReportTimeElapsed (
			);

		//! Report timer value.
		void ReportTimerValue (
			const MTIMER& timer
			) { v_ReportTimerValue(timer); }

		//! Reset cancelled state.
		//! Does not reset cancelled state of already attached contexts.
		void ResetCancelled (
			);

		//! Reset and start timer.
		void RestartTimer (
			) { m_Timer.Clear(); m_Timer.Start(); }

		//! Set instance to be active status manager.
		void SetAsActive (
			) { SetActiveManager(this); }

		//! Set whether cancel is enabled.
		//! \internal Used by STATUSCONTEXT, not usually called directly.
		//! For some jobs/steps, cancellation may be disabled.  This method
		//! provides a way to notify the user that cancellation is not allowed.
		//! This should be done sparingly.  By default, cancellation is assumed to be enabled.
		//! This will modify the "disable count" and only call v_SetCancelEnabled() when state changes.
		//  (Will become private when legacy wrappers are removed)
		void SetCancelEnabled (
			bool enabled
			);

	protected:
		typedef MILIST<MISTATUSCONTEXT*> CONTEXTLIST;

		//! Base for private manager-specific info to store with context.
		class CLASSLIBEXPORT CONTEXT_PRIV {
			public:
				virtual ~CONTEXT_PRIV () = 0;
			};

		//! Clear current context list.
		void ClearContextList (
			);

		//! Get list of pointers to attached contexts.
		const CONTEXTLIST& GetContextList (
			MITHREAD::CRITICALSECTION::LOCK& Lock		//!< Must be a LOCK that is not assigned to a Critical section, This method will assign it.
			) const;

		//! Get ID of thread in which manager was created.
		MISYSTEM::THREAD_ID GetCreatorThreadID (
			) const { return (m_CreatorThreadID); }

		//! Determine if "cancelled" state has been set.
		bool IsCancelSet (
			) const { return (m_IsCancelled); }

		//! Report message.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		void ReportMessage (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			) { v_ReportMessage(context,msgstr); }

		//! Set "cancelled" state.
		//! When cancelled state is first set, all attached contexts will be set as cancelled.
		void SetCancelled (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class MISTATUSCONTEXT;
		CONTEXTLIST m_ContextList;
		mutable MITHREAD::CRITICALSECTION m_ContextListSync;
		MISYSTEM::THREAD_ID m_CreatorThreadID;
		UINT32 m_CancelDisableCount;
		bool m_IsCancelled;
		MTIMER m_Timer;
		double m_LastElapsed;

		//! Create status manager using previously assigned creator.
		//! Caller is responsible for properly destroying manager.
		//! May return null if no creator assigned (typically for worker threads).
		//! Used internally by STATUSCONTEXT, not usually called directly.
		static MISTATUSMANAGER* CreateManager ();

		//! Attach context to manager.
		//! Used internally by MISTATUSCONTEXT, not called directly.
		void ContextAttach (
			MISTATUSCONTEXT *pContext
			);

		//! Detach context from manager.
		//! Used internally by MISTATUSCONTEXT, not called directly.
		void ContextDetach (
			MISTATUSCONTEXT *pContext
			);

		//! Create private manager-specific information for context.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		CONTEXT_PRIV* ContextPrivCreate (
			MISTATUSCONTEXT& Context
			) { return (v_ContextPrivCreate(Context)); }

		//! Report that task is beginning.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		void ReportBegin (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			) { v_ReportBegin(context,msgstr); }

		//! Report that task is ending.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		void ReportEnd (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr,
			const MISTRING& timerstr
			) { v_ReportEnd(context,msgstr,timerstr); }

		//! Report progress.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		void ReportProgress (
			MISTATUSCONTEXT& context,
			double pctdone
			) { v_ReportProgress(context,pctdone); }

		//! Update user interface.
		//! Used internally by STATUSCONTEXT, not usually called directly.
		void UpdateUI (
			) { v_UpdateUI(); }

		// Unimplemented.
		MISTATUSMANAGER (const MISTATUSMANAGER&);
		MISTATUSMANAGER& operator= (const MISTATUSMANAGER&);
		#endif	// GENERATING_DOXYGEN_OUTPUT

		//! \name Overridables
		//!@{

		//! Create private manager-specific information for context.
		//! Default implementation needs no private info and so returns NULL.
		virtual CONTEXT_PRIV* v_ContextPrivCreate (
			MISTATUSCONTEXT& Context
			);

		//! Determine if cancel has been requested.
		//! Default implementation always returns false.
		virtual bool v_IsCancelled (
			);

		//! Report task beginning.
		//! Default implementation calls ReportMessage.
		virtual void v_ReportBegin (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			);

		//! Report that task is ending.
		//! Default implementation does nothing.
		virtual void v_ReportEnd (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr,
			const MISTRING& timerstr
			);

		//! Report message.
		//! Default implementation does nothing.
		virtual void v_ReportMessage (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			);

		//! Report progress.
		//! Default implementation does nothing.
		virtual void v_ReportProgress (
			MISTATUSCONTEXT& context,
			double pctdone
			);

		//! Report timer value.
		//! Default implementation does nothing.
		virtual void v_ReportTimerValue (
			const MTIMER& timer
			);

		//! Reset cancelled state.
		//! Default implementation does nothing.
		virtual void v_ResetCancelled (
			);

		//! Set whether cancel is enabled.
		//! Only needs to be implemented to update user interface, as
		//! state is retained by STATUSMANAGER itself.
		virtual void v_SetCancelEnabled (
			bool enabled
			);

		//! Update user interface display.
		//! Default implementation does nothing.
		virtual void v_UpdateUI (
			);

		//!@}
	};

//===================================================================================================================

//! Status logger, provides automatic recording of messages to session log.
class CLASSLIBEXPORT MISTATUSLOGGER : public MISTATUSMANAGER {
	public:

		//! Default constructor.
		MISTATUSLOGGER ();

		//! Destructor.
		virtual ~MISTATUSLOGGER ();

		//! Disable logging
		//! Each call must be matched by call to SetLogEnabled to re-enable logging.
		void SetLogDisabled (
			) { ++m_LogDisableCount; }

		//! Enable logging.
		void SetLogEnabled (
			) { if (m_LogDisableCount) --m_LogDisableCount; }

	protected:

		//! Report task beginning.
		//! Default implementation calls ReportMessage and logs message.
		//! Derived class should call base implementation to do logging.
		virtual void v_ReportBegin (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			);

		//! Report that task is ending.
		//! Formats strings together and logs.
		//! Derived class should call base implementation to do logging.
		virtual void v_ReportEnd (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr,
			const MISTRING& timerstr
			);

		//! Report message.
		//! Derived class should call base implementation to do logging.
		virtual void v_ReportMessage (
			MISTATUSCONTEXT& context,
			const MISTRING& msgstr
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_LogDisableCount;
		// Unimplemented.
		MISTATUSLOGGER (const MISTATUSLOGGER&);
		MISTATUSLOGGER& operator= (const MISTATUSLOGGER&);
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Status context, allows cancel checking and progress reporting.
//!
//! Do not inherit from this class.
//!
//! This class is designed to be constructed on the stack at the beginning of a "task so
//! when it goes out of scope the ending report will be made.
//!
//! Example using active status manager:
//!
//!	\code
//!	ERRVALUE DoTask (
//!		) {
//!		ERRVALUE err;
//!		MISTATUSCONTEXT context(TEXTID_TaskName);
//!		context.ReportProgressBegin(NumSteps);
//!		for (int step = 0; (step < NumSteps); ++step) {
//!			if ((err = context.ReportProgressIncrement()) < 0) return (err);
//!			}
//!		return (0);
//!		}
//!	\endcode
//!
//! Example using specified status manager and percentage:
//!
//!	\code
//!	ERRVALUE DoTask (
//!		MISTATUSMANAGER *pStatusMgr
//!		) {
//!		ERRVALUE err;
//!		MISTATUSCONTEXT context(pStatusMgr,TEXTID_TaskName);
//!		for (int step = 0; (step < NumSteps); ++step) {
//!			if ((err = context.ReportProgress(100.0*step/NumSteps)) < 0) return (err);
//!			}
//!		return (0);
//!		}
//!	\endcode
class CLASSLIBEXPORT MISTATUSCONTEXT {
	public:

		//! Construct without status manager.
		//! Use Create() to setup with status manager.
		MISTATUSCONTEXT ();

		//! Construct using specified status manager.
		MISTATUSCONTEXT (
			MISTATUSMANAGER *pStatusMgr,		//!< Status manager, if 0 then context will not report to any manager
			const MISTRING& taskstr,			//!< Task description
			bool AllowCancel = true				//!< Whether to allow cancel for this task
			);

		//! Construct using active status manager.
		MISTATUSCONTEXT (
			const MISTRING& taskstr,			//!< Task description
			bool AllowCancel = true,			//!< Whether to allow cancel for this task
			bool CreateMgrIfNoActive = false	//!< Create own status manager if no active one set
			);

		//! Destructor.
		~MISTATUSCONTEXT (
			);

		//! Create with specified status manager.
		//! Should only be used if default constructor was also used.
		ERRVALUE Create (
			MISTATUSMANAGER *pStatusMgr,		//!< Status manager, if 0 then context will not report to any manager
			const MISTRING& taskstr,			//!< Task description
			bool AllowCancel = true				//!< Whether to allow cancel for this task
			) { return (CreateLow(pStatusMgr,taskstr,AllowCancel,false)); }

		//! Create with active status manager.
		//! Should only be used if default constructor was also used.
		ERRVALUE Create (
			const MISTRING& taskstr,			//!< Task description
			bool AllowCancel = true,			//!< Whether to allow cancel for this task
			bool CreateMgrIfNoActive = false	//!< Create own status manager if no active one set
			) { return (CreateLow(MISTATUSMANAGER::GetActiveManager(),taskstr,AllowCancel,CreateMgrIfNoActive)); }

	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Detach from manager.
		//! For internal use only.
		void DetachMgr (
			) { m_pStatusMgr = 0; m_IsCancelled = true; }
	#endif

		//! Get attached status manager.
		//! \internal For implementation use only.
		MISTATUSMANAGER* GetManager (
			) const { return (m_pStatusMgr); }

		//! Get private manager-specific information.
		//! \internal For status manager implementations.
		MISTATUSMANAGER::CONTEXT_PRIV* GetPriv (
			) const { return (m_pPriv); }

		//! Determine if attached to a status manager.
		bool HasManager (
			) const { return (m_pStatusMgr != 0); }

		//! Check if operation has been cancelled.
		bool IsCancelled (
			) { return (m_IsCancelled || (m_IsCancelled = (m_pStatusMgr != 0 && m_pStatusMgr->IsCancelled()))); }

		//! Report message.
		//! Should not be used to report start/end of task, that is handled by ctor/dtor.
		ERRVALUE ReportMessage (
			const MISTRING& msgstr
			) { if (m_pStatusMgr != 0) m_pStatusMgr->ReportMessage(*this,msgstr); return (TestCancelled()); }

		//! Report progress by percentage.
		ERRVALUE ReportProgress (
			double pctdone = -1					//!< Percentage complete, -1 indicates indeterminant
			) { ReportProgressNoCancel(pctdone); return (TestCancelled()); }

		//! Report progress by percentage without cancel check.
		void ReportProgressNoCancel (
			double pctdone = -1					//!< Percentage complete, -1 indicates indeterminant
			) { if (m_pStatusMgr != 0) m_pStatusMgr->ReportProgress(*this,pctdone); }

		//! Begin report of progress with known number of steps.
		ERRVALUE ReportProgressBegin (
			UINT32 NumSteps						//!< Total number of steps for task
			);

		//! Increment report of progress with known number of steps.
		ERRVALUE ReportProgressIncrement (
			INT32 IncSteps = 1					//!< Number of steps to increment by
			) { ReportProgressIncrementNoCancel(IncSteps); return (TestCancelled()); }

		//! Increment report of progress without check for cancel.
		void ReportProgressIncrementNoCancel (
			INT32 IncSteps = 1					//!< Number of steps to increment by
			) { m_CurStep += IncSteps; if (m_pStatusMgr != 0) m_pStatusMgr->ReportProgress(*this,100.0*m_CurStep/m_NumSteps); }

		//! Set that context is cancelled.
		//! \internal For use by base status manager implementation.
		void SetCancelled (
			) { m_IsCancelled = true; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTATUSMANAGER *m_pStatusMgr;
		MISTATUSMANAGER::CONTEXT_PRIV *m_pPriv;
		MISTRING m_taskstr;
		MTIMER m_Timer;
		UINT32 m_NumSteps;
		INT32 m_CurStep;
		bool m_AllowCancel;
		bool m_OwnStatusMgr;
		bool m_IsCancelled;

		ERRVALUE CreateLow (MISTATUSMANAGER *pStatusMgr, const MISTRING& taskstr, bool AllowCancel, bool CreateMgrIfNoneSet);
		void Initialize (bool CreateMgrIfNoneSet);
		ERRVALUE TestCancelled (
			) { return ((m_AllowCancel && IsCancelled()) ? EUserCancel : 0); }

		// Unimplemented.
		MISTATUSCONTEXT (const MISTATUSCONTEXT&);
		MISTATUSCONTEXT& operator= (const MISTATUSCONTEXT&);
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_MISTATUS_H
