/**
 *	\file mi32/unsignedset.h
 *	\brief Definitions for UNSIGNEDSET class.
 *
 *	\if NODOC
 *	$Id: unsignedset.h_v 1.14 2011/11/03 16:54:56 mju Exp $
 *
 * $Log: unsignedset.h_v $
 * Revision 1.14  2011/11/03 16:54:56  mju
 * Add decompressForContains.
 *
 * Revision 1.13  2011/07/21 16:24:14  mju
 * Add contains with uint16.
 *
 * Revision 1.12  2011/07/21 16:21:15  mju
 * Add addSingle with uint16.
 *
 * Revision 1.11  2010/08/18 21:06:07  mju
 * Add overrides with int32/64 parms.
 *
 * Revision 1.10  2010/04/26 15:49:20  mju
 * Add setContains.
 *
 * Revision 1.9  2010/04/22 13:17:24  mju
 * Add iterator_intersection.
 *
 * Revision 1.8  2010/04/21 21:26:08  mju
 * Add setEnd methods for iterators.
 *
 * Revision 1.7  2010/04/21 17:24:03  mju
 * Add exchange method.
 *
 * Revision 1.6  2010/04/19 15:16:10  mju
 * Add getBitSet.
 *
 * Revision 1.5  2010/04/19 13:37:05  mju
 * Change toggleSingle to return new state.
 *
 * Revision 1.4  2010/04/19 13:02:44  mju
 * Rename attach as AttachCompressed and assign from buffer as LoadCompressed.
 * Implement assign from bitset.
 *
 * Revision 1.3  2010/04/16 21:13:14  mju
 * Add 32-bit specific methods.
 * Add Intersect, UnionExclusive, Remove, Toggle and more.
 *
 * Revision 1.2  2010/04/02 17:59:40  mju
 * Add assign from unsignedset and simple_array.
 *
 * Revision 1.1  2010/04/02 16:10:57  mju
 * Initial revision
 *
 * \endif
***/

#ifndef INC_MI32_UNSIGNEDSET_H
#define INC_MI32_UNSIGNEDSET_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_BITSET_H
#include <mi32/bitset.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


//=====================================================================================================================
//! Set of unsigned values, stored in compressed form.
class CLASSLIBEXPORT UNSIGNEDSET {
	public:

		//! Iterator to step forward through UINT32 items in set.
		class CLASSLIBEXPORT ITERATOR32 {
			public:

				//! Construct from set.
				//! If items in set exceed 32-bit range iterator will be set to end, use ITERATOR64 for that case.
				explicit ITERATOR32 (
					const UNSIGNEDSET& Set
					) :
					m_Set(Set)
					{ Reset(); }

				//! Dereference operator, returns item number in set.
				UINT32 operator* (
					) const { return (m_Num); }

				//! Cast operator to UINT32.
				operator UINT32 (
					) const { return (m_Num); }

				//! Pre-increment to next item in set.
				ITERATOR32& operator++ ();

				//! Determine if have reached end of set.
				bool IsEnd () const
					{ return (m_Num == UINT32_MAX); }

				//! Reset iterator to first item in set.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_Num = UINT32_MAX; }

				//! Skip ahead by current count, returns 0 if no current count.
				UINT32 SkipCount ();

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				const UNSIGNEDSET& m_Set;
				const UINT8 *m_pData;
				const UINT8 *m_pEnd;
				UINT32 m_Num;
				UINT32 m_CountLeft;
	   		#endif // GENERATING_DOXYGEN_OUTPUT
			};
		
		//! Iterator to step forward through UINT64 items in set.
		class CLASSLIBEXPORT ITERATOR64 {
			public:

				//! Construct from set.
				explicit ITERATOR64 (
					const UNSIGNEDSET& Set
					) :
					m_Set(Set)
					{ Reset(); }

				//! Dereference operator, returns item number in set.
				UINT64 operator* (
					) const { return (m_Num); }

				//! Cast operator to UINT64.
				operator UINT64 (
					) const { return (m_Num); }

				//! Pre-increment to next item in set.
				ITERATOR64& operator++ ();

				//! Determine if have reached end of set.
				bool IsEnd () const
					{ return (m_Num == UINT64_MAX); }

				//! Reset iterator to first item in set.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_Num = UINT64_MAX; }

				//! Skip ahead by current count, returns 0 if no current count.
				UINT32 SkipCount ();

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				const UNSIGNEDSET& m_Set;
				const UINT8 *m_pData;
				const UINT8 *m_pEnd;
				UINT64 m_Num;
				UINT32 m_CountLeft;
	   		#endif // GENERATING_DOXYGEN_OUTPUT
			};
		
		//! Iterator to step forward through UINT32 ranges of items in set.
		class CLASSLIBEXPORT ITERATOR_RANGE32 {
			public:

				//! Construct from set.
				//! If items in set exceed 32-bit range iterator will be set to end, use ITERATOR_RANGE64 for that case.
				explicit ITERATOR_RANGE32 (
					const UNSIGNEDSET& Set
					) :
					m_itNext(Set)
					{ Reset(); }

				//! Pre-increment to next range in set.
				ITERATOR_RANGE32& operator++ ();

				//! Get first item in range.
				UINT32 GetFirst () const
					{ return (m_NumFirst); }

				//! Get last item in range.
				UINT32 GetLast () const
					{ return (m_NumLast); }

				//! Determine if have reached end of set.
				bool IsEnd () const
					{ return (m_NumFirst == UINT32_MAX); }

				//! Reset iterator to first range in set.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_NumFirst = m_NumLast = UINT32_MAX; }

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				ITERATOR32 m_itNext;
				UINT32 m_NumFirst;
				UINT32 m_NumLast;
	   		#endif // GENERATING_DOXYGEN_OUTPUT
			};
		
		//! Iterator to step forward through UINT64 ranges of items in set.
		class CLASSLIBEXPORT ITERATOR_RANGE64 {
			public:

				//! Construct from set.
				explicit ITERATOR_RANGE64 (
					const UNSIGNEDSET& Set
					) :
					m_itNext(Set)
					{ Reset(); }

				//! Pre-increment to next range in set.
				ITERATOR_RANGE64& operator++ ();

				//! Get first item in range.
				UINT64 GetFirst () const
					{ return (m_NumFirst); }

				//! Get last item in range.
				UINT64 GetLast () const
					{ return (m_NumLast); }

				//! Determine if have reached end of set.
				bool IsEnd () const
					{ return (m_NumFirst == UINT64_MAX); }

				//! Reset iterator to first range in set.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_NumFirst = m_NumLast = UINT64_MAX; }

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				ITERATOR64 m_itNext;
				UINT64 m_NumFirst;
				UINT64 m_NumLast;
	   		#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Iterator to step through UINT32 items in intersection of two sets.
		class CLASSLIBEXPORT ITERATOR_INTERSECTION32 {
			public:

				//! Construct from sets.
				//! If items in set exceed 32-bit range iterator will be set to end, use 64-bit iterator for that case.
				explicit ITERATOR_INTERSECTION32 (
					const UNSIGNEDSET& set1,
					const UNSIGNEDSET& set2
					) :
					m_it1(set1),
					m_it2(set2)
					{ Reset(); }

				//! Dereference operator, returns item number in set.
				UINT32 operator* (
					) const { return (m_Num); }

				//! Cast operator to UINT32.
				operator UINT32 (
					) const { return (m_Num); }

				//! Pre-increment to next item.
				ITERATOR_INTERSECTION32& operator++ ();

				//! Determine if have reached end.
				bool IsEnd () const
					{ return (m_Num == UINT32_MAX); }

				//! Reset iterator to first item.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_Num = UINT32_MAX; }

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				ITERATOR_RANGE32 m_it1;
				ITERATOR_RANGE32 m_it2;
				UINT32 m_Num;
				#endif
			};
		
		//! Iterator to step through UINT64 items in intersection of two sets.
		class CLASSLIBEXPORT ITERATOR_INTERSECTION64 {
			public:

				//! Construct from sets.
				//! If items in set exceed 64-bit range iterator will be set to end, use 64-bit iterator for that case.
				explicit ITERATOR_INTERSECTION64 (
					const UNSIGNEDSET& set1,
					const UNSIGNEDSET& set2
					) :
					m_it1(set1),
					m_it2(set2)
					{ Reset(); }

				//! Dereference operator, returns item number in set.
				UINT64 operator* (
					) const { return (m_Num); }

				//! Cast operator to UINT64.
				operator UINT64 (
					) const { return (m_Num); }

				//! Pre-increment to next item.
				ITERATOR_INTERSECTION64& operator++ ();

				//! Determine if have reached end.
				bool IsEnd () const
					{ return (m_Num == UINT64_MAX); }

				//! Reset iterator to first item.
				void Reset ();

				//! Set iterator to end.
				void SetEnd ()
					{ m_Num = UINT64_MAX; }

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				ITERATOR_RANGE64 m_it1;
				ITERATOR_RANGE64 m_it2;
				UINT64 m_Num;
				#endif
			};
		
		//! Default constructor.
		UNSIGNEDSET ();

		//! Destructor.
		~UNSIGNEDSET ();

		//! Add range of item numbers to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddRange (
			UINT32 min,								//!< Minimum item number
			UINT32 max								//!< Maximum item number
			);

		//! Add range of item numbers to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddRange (
			INT32 min,								//!< Minimum item number
			INT32 max								//!< Maximum item number
			) { return (AddRange(static_cast<UINT32>(min),static_cast<UINT32>(max))); }

		//! Add range of 64-bit item numbers to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddRange (
			UINT64 min,								//!< Minimum item number
			UINT64 max								//!< Maximum item number
			);

		//! Add range of 64-bit item numbers to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddRange (
			INT64 min,								//!< Minimum item number
			INT64 max								//!< Maximum item number
			) { return (AddRange(static_cast<UINT64>(min),static_cast<UINT64>(max))); }

		//! Add single item number to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddSingle (
			UINT16 num								//!< Item number to add
			) { return (AddSingle(static_cast<UINT32>(num))); }

		//! Add single item number to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddSingle (
			UINT32 num								//!< Item number to add
			) { return (AddRange(num,num)); }

		//! Add single item number to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddSingle (
			INT32 num								//!< Item number to add
			) { return (AddRange(num,num)); }

		//! Add single 64-bit item number to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddSingle (
			UINT64 num								//!< Item number to add
			) { return (AddRange(num,num)); }

		//! Add single 64-bit item number to set.
		//! To maintain set in "compressed" form, items must be added in ascending order.
		ERRVALUE AddSingle (
			INT64 num								//!< Item number to add
			) { return (AddRange(num,num)); }

		//! Assign set from another.
		ERRVALUE Assign (
			const UNSIGNEDSET& set				//!< Set to assign from
			);

		//! Assign from BITSET.
		ERRVALUE Assign (
			const BITSET_UNOWNED& bitset		//!< Set to assign from
			);

		//! Attach set to compressed data, taking ownership.
		//! Any iterators referencing the set will become invalid.
		ERRVALUE AttachCompressed (
			UINT8 *& pData,						//!< Pointer to compressed data, returned as NULL
			UINT32 BufSize							//!< Size of data in bytes
			);

		//! Attach set to compressed data, taking ownership.
		//! Any iterators referencing the set will become invalid.
		ERRVALUE AttachCompressed (
			SIMPLE_ARRAY<UINT8>& Data			//!< Compressed data array, will be exchanged with existing data.
			);

		//! Clear the set.
		//! Any iterators referencing the set will become invalid.
		void Clear ();

		//! Compress this set if not already.
		ERRVALUE Compress ();

		//! Determine if set contains specified item number.
		//! This may be very inefficient for compressed sets, use of iterator is generally faster if multiple items are being tested.
		//! If use of iterator is not practical and multiple calls to Contains might be made, using DecompressForContains can improve performance.
		bool Contains (
			UINT16 ItemNum
			) const { return (Contains(static_cast<UINT32>(ItemNum))); }

		//! Determine if set contains specified item number.
		//! This may be very inefficient for compressed sets, use of iterator is generally faster if multiple items are being tested.
		//! If use of iterator is not practical and multiple calls to Contains might be made, using DecompressForContains can improve performance.
		bool Contains (
			UINT32 ItemNum
			) const;

		//! Determine if set contains specified item number.
		//! This may be very inefficient for compressed sets, use of iterator is generally faster if multiple items are being tested.
		//! If use of iterator is not practical and multiple calls to Contains might be made, using DecompressForContains can improve performance.
		bool Contains (
			INT32 ItemNum
			) const { return (Contains(static_cast<UINT32>(ItemNum))); }

		//! Determine if set contains specified 64-bit item number.
		//! This may be very inefficient for compressed sets, use of iterator is generally faster if multiple items are being tested.
		//! If use of iterator is not practical and multiple calls to Contains might be made, using DecompressForContains can improve performance.
		bool Contains (
			UINT64 ItemNum
			) const;

		//! Determine if set contains specified 64-bit item number.
		//! This may be very inefficient for compressed sets, use of iterator is generally faster if multiple items are being tested.
		//! If use of iterator is not practical and multiple calls to Contains might be made, using DecompressForContains can improve performance.
		bool Contains (
			INT64 ItemNum
			) const { return (Contains(static_cast<UINT64>(ItemNum))); }

		//! Count number of items in set.
		UINT64 CountItems () const;

		//! Decompress this set if not already.
		ERRVALUE Decompress ();

		//! Decompress if will make more efficient for multiple calls to Contains().
		ERRVALUE DecompressForContains ();

		//! Exchange contents of two sets (efficiently).
		//! Any iterators referencing either set will become invalid.
		void UNSIGNEDSET::Exchange (
			UNSIGNEDSET& other						//!< Set to exchange with
			);

		//! Get as BITSET.
		ERRVALUE GetBitSet (
			BITSET& bitset,				//!< Bit set to contain results
			UINT32 NumEntries				//!< Number of entries to size bitset to
			) const;

		//! Get compressed data array.
		//! Returned array will be empty if set is not compressed.
		const SIMPLE_ARRAY<UINT8>& GetCompressed ();

		//! Get first item in set.
		UINT64 GetFirstItem () const
			{ return (m_FirstItem); }

		//! Get last item in set.
		UINT64 GetLastItem () const
			{ return (m_LastItem); }

		//! Intersect this set with another, resulting in set containing items that exist in BOTH sets.
		//! This set may become decompressed as a result of this operation.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE Intersect (
			const UNSIGNEDSET& rhs
			);

		//! Determine if set is "compressed".
		bool IsCompressed () const
			{ return (!m_SpanAllInSet.HasEntries()); }

		//! Determine if set is empty.
		bool IsEmpty () const
			{ return (m_Data.IsEmpty() && !m_SpanAllInSet.HasEntries()); }

		//! Load set from compressed data.
		//! Any iterators referencing the set will become invalid.
		ERRVALUE LoadCompressed (
			const UINT8 *pData,					//!< Pointer to compressed data
			UINT32 BufSize							//!< Size of data in bytes
			);

		//! Load set from compressed data array.
		//! Any iterators referencing the set will become invalid.
		ERRVALUE LoadCompressed (
			const SIMPLE_ARRAY<UINT8>& Data	//!< Array of compressed data
			) { return (LoadCompressed(static_cast<const UINT8*>(Data),Data.GetNumItems())); }

		//! Remove range of item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveRange (
			UINT32 min,								//!< Minimum item number
			UINT32 max								//!< Maximum item number
			);

		//! Remove range of item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveRange (
			INT32 min,								//!< Minimum item number
			INT32 max								//!< Maximum item number
			) { return (RemoveRange(static_cast<UINT32>(min),static_cast<UINT32>(max))); }

		//! Remove range of 64-bit item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveRange (
			UINT64 min,								//!< Minimum item number
			UINT64 max								//!< Maximum item number
			);

		//! Remove range of 64-bit item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveRange (
			INT64 min,								//!< Minimum item number
			INT64 max								//!< Maximum item number
			) { return (RemoveRange(static_cast<UINT64>(min),static_cast<UINT64>(max))); }

		//! Remove set of items from this set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveSet (
			const UNSIGNEDSET& set				//!< Set of items to remove
			);

		//! Remove single item number to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveSingle (
			UINT32 num								//!< Item number to remove
			) { return (RemoveRange(num,num)); }

		//! Remove single item number to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveSingle (
			INT32 num								//!< Item number to remove
			) { return (RemoveRange(num,num)); }

		//! Remove single 64-bit item number to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveSingle (
			UINT64 num								//!< Item number to remove
			) { return (RemoveRange(num,num)); }

		//! Remove single 64-bit item number to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE RemoveSingle (
			INT64 num								//!< Item number to remove
			) { return (RemoveRange(num,num)); }

		//! Set whether set contains specified 32-bit item.
		//! Any iterators referencing this set may become invalid.
		ERRVALUE SetContains (
			UINT32 num,								//!< Item number
			bool inset								//!< True to add to set, false to remove
			);

		//! Set whether set contains specified 64-bit item.
		//! Any iterators referencing this set may become invalid.
		ERRVALUE SetContains (
			UINT64 num,								//!< Item number
			bool inset								//!< True to add to set, false to remove
			);

		//! Toggle range of item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE ToggleRange (
			UINT32 min,								//!< Minimum item number
			UINT32 max								//!< Maximum item number
			);

		//! Toggle range of 64-bit item numbers to set.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE ToggleRange (
			UINT64 min,								//!< Minimum item number
			UINT64 max								//!< Maximum item number
			);

		//! Toggle single item number in set.
		//! Any iterators referencing this set will become invalid.
		//! @return 1 if item added to set, 0 if removed, < 0 if error
		int ToggleSingle (
			UINT64 num								//!< Item number to toggle
			);

		//! Union this set with another, resulting in set containing items that exist in EITHER set.
		//! This set may become decompressed as a result of this operation.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE Union (
			const UNSIGNEDSET& rhs
			);

		//! Exclusive union of this set with another, resulting in set containint items that exist in EITHER set but not both.
		//! This set may become decompressed as a result of this operation.
		//! Any iterators referencing this set will become invalid.
		ERRVALUE UnionExclusive (
			const UNSIGNEDSET& rhs
			);

	private:
	 	#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT8> m_Data;
		UINT64 m_FirstItem;
		UINT64 m_LastItem;
		UINT64 m_LastCount;
		UINT32 m_LastCountIndex;
		UINT32 m_EndIndex;
		BITSET m_SpanAllInSet;
		SIMPLE_ARRAY<UINT32*> m_SpanData;
		SIMPLE_ARRAY<UINT32*> m_SpanPool;

		ERRVALUE AppendCount (UINT32 value);
		ERRVALUE AppendCount (UINT64 value);
		ERRVALUE AppendSkip (UINT32 value);
		ERRVALUE AppendSkip (UINT64 value);
		void FindNewFirstItemUncompressed ();
		void FindNewLastItemUncompressed ();
		void FreeSpans ();
		ERRVALUE GetSpan (UINT32 Span, bool initialize = true);
		ERRVALUE LoadFinish ();
		void ReleaseSpanToPool (UINT32 Span);
		ERRVALUE RangeAddCompressed (UINT32 First, UINT32 Last);
		ERRVALUE RangeAddCompressed (UINT64 First, UINT64 Last);
		ERRVALUE RangeAddUncompressed (UINT32 First, UINT32 Last);
		ERRVALUE RangeAddUncompressed (UINT64 First, UINT64 Last);
		ERRVALUE RangeRemoveUncompressed (UINT32 First, UINT32 Last);
		ERRVALUE RangeRemoveUncompressed (UINT64 First, UINT64 Last);
		ERRVALUE RangeToggleUncompressed (UINT32 First, UINT32 Last);
		ERRVALUE RangeToggleUncompressed (UINT64 First, UINT64 Last);
		ERRVALUE SpanClearRange (UINT32 Span, UINT32 MinPos, UINT32 MaxPos);
		ERRVALUE SpanSetRange (UINT32 Span, UINT32 MinPos, UINT32 MaxPos);
		ERRVALUE SpanToggleRange (UINT32 Span, UINT32 MinPos, UINT32 MaxPos);

		// Unimplemented.
		UNSIGNEDSET (const UNSIGNEDSET&);
		UNSIGNEDSET& operator= (const UNSIGNEDSET&);
	   #endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

#undef	CLASSLIBEXPORT

#endif		// INC_MI32_UNSIGNEDSET_H
