/******************************************************************************
 *
 * \file sml/class.h
 * \brief SML CLASS
 *
 * \if NODOC
 * $Log: class.h_v $
 * Revision 1.18  2011/10/12 14:29:12  mju
 * Remove redundant hdrs.
 *
 * Revision 1.17  2011/08/19 22:42:31  bcolombe
 * Add bRequired optional param to AddMethods
 *
 * Revision 1.16  2011/08/19 21:39:51  bcolombe
 * Add GenerateBuilderText
 *
 * Revision 1.15  2008/01/15 19:06:10  bcolombe
 * Fix for mac SetConstructor with no params
 *
 * Revision 1.14  2008/01/15 15:42:36  bcolombe
 * Fix Mac warning 'there are no arguments to SetConstructor that depend on a template paramater'
 *
 * Revision 1.13  2008/01/15 15:32:26  bcolombe
 * Moved smlautoclass destructor to own method SetDestructor
 *
 * Revision 1.12  2008/01/11 00:06:44  bcolombe
 * Change to destructors for smlautoclass
 *
 * Revision 1.11  2008/01/04 22:20:41  bcolombe
 * Added new AddMethodOldCB
 *
 * Revision 1.10  2007/08/03 15:44:13  dwilliss
 * eliminated deprecated macros which are no longer used
 *
 * Revision 1.9  2007/08/03 14:33:56  dwilliss
 * added method to set just destructor
 *
 * Revision 1.8  2007/05/31 20:36:35  drb
 * Added autoclass basic constructors
 *
 * Revision 1.7  2007/05/17 19:42:59  dwilliss
 * Restriction allows multiple products now
 *
 * Revision 1.6  2007/05/17 19:28:50  dwilliss
 * made GetRestrictions() public instead of protected
 *
 * Revision 1.5  2007/05/17 17:59:14  dwilliss
 * Changed how we handle restrictions
 *
 * Revision 1.4  2007/05/15 22:19:26  dwilliss
 * keep track of product codes
 *
 * Revision 1.3  2007/05/08 15:45:24  dwilliss
 * SML namespace
 *
 * Revision 1.2  2007/05/07 15:43:36  dwilliss
 * *** empty log message ***
 *
 * Revision 1.1  2007/05/04 23:02:32  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_CLASS_H
#define INC_SML_CLASS_H

#ifndef	INC_SML_CLASSMEMBER_H
	#include <sml/classmember.h>
#endif

#ifndef	INC_SML_CONTEXTTYPE_H
	#include <sml/contexttype.h>
#endif

#ifndef	INC_SML_FUNCTION_H
	#include <sml/function.h>
#endif

#ifndef	INC_SML_RESTRICTION_H
	#include <sml/restriction.h>
#endif

#ifndef INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


namespace SML {
	class CLASSAUTOREG;
	class CLASSHASH;
	class AUTOCLASS;

/*****************************************************************************/
/**
*	Structure for defining a class in SML.
*/
class CLASS {
	public:
		const char *name;				//!<  Name of this type class
		const char *BaseClassName;
		CLASSMEMBER *members;

		//! Constructors will be called in base to derived class order.  Data
		//! passed in will be NULL or preallocated and cleared to the size
		//! specified by the largest derived class.  If no size is specified,
		//! the base class's constructor will get a NULL pointer and is expected
		//! to allocate the data itself.  Derived classs constructors are expected
		//! to use the data passed and return it.
		void *(*Constructor)(void *data, CONTEXT *context, CLASS *);

		//! Destructors are called in derived to base class order. It's up to you
		//! to make sure that if a constructor allocated the data, a destructor
		//! frees it.  But make sure that only the lowest level destructor frees
		//! it or you will illop.
		//! If the data was not allocated by a constructor (ie, a size was given
		//! in one of the classes, then the data will be free'd automatically
		void *(*Destructor)(void *data, CONTEXT *context, CLASS*);

		//! Copy a class.
		//! If the size field in SML::CLASS is 0, then the copy constructor must
		//! malloc and return a copy of src (dest parameter is ignored)
		//! This is only used this way in one place currently. For copying colormaps
		//! If size is non-zero, src should be coppied into dest and return dest.
		//! This is used mostly for assigning into an array of classes.
		//! If there is no copy constructor, an array element assignment will
		//! use memcpy()
		void *(*CopyConstructor)(void *src, CONTEXT *context, CLASS*, void* dest);

		//! Called to create SMLBuilder text for class
		void GenerateBuilderText (
			MISTRING& code, 
			const MISTRING& name, 
			const MISTRINGLIST& ParamList
			) { GenerateBuilderTextFunc(code, name, ParamList); }

		//! Generate code for SML Builder
		void (*GenerateBuilderTextFunc)(MISTRING& code, const MISTRING& name, const MISTRINGLIST& ParamList);

		int size;
		INT32 CreateDate;					//!< 0 means some time before 19980603
		INT32 ModDate;
		const char *description;		//!< Default description which is use to generate the documentation
		CLASS *next;			//!< Linked list of registered classes
		CLASS *BaseClass;	//!< Leave this 0.  InstallClass will look up BaseClassName and set it.
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT8 Deprecated;		//!< 1 if the class is deprecated
		#endif // GENERATING_DOXYGEN_OUTPUT
		INT8 NotInWin32Native;
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT8 SpareByte1;		//!< Don't need to document spare bytes
		INT8 SpareByte2;		//!< Don't need to document spare bytes
		#endif // GENERATING_DOXYGEN_OUTPUT
		int OffsetToNext;					//!< If class is a linked list, offset to the "next" pointer (-1 if not)
		void *privdata;
		int sizeofSMLCLASS;
		int sizeofSMLCLASSMEMBER;
		void (*UninstallClass)(CLASS*);	//! If not NULL and sizeofSMLCLASS allows for it, this will be called when the context is destroyed
		const RESTRICTION* m_Restriction;

	};


/***************************************************************************/

//! A helper class to simplify installing real C++ classes into SML.
//!
//!	Example usage
//!
//! @code
//! // Declare that this class is only to be used in SML, not queries.
//! // Note, an SML::AUTOCLASS must exist for the life of the process.
//! // and is usually created statically.  This causes it to be constructed
//! // at the beginning of the program and automatically registers itself.
//! static SML::AUTOCLASS_NEW_T<FOOBAR> s_AutoClass(&FOOBAR::SetupSMLClass, SML::CONTEXTTYPE_SML);
//!
//! // [ static method of FOOBAR ]
//! void FOOBAR::SetupSMLClass (
//!		) {
//!		typedef void (FOOBAR::*VOIDMETHOD)();
//!		s_AutoClass.Init("FOOBAR", NULL);
//!
//!		// Parametes for a method that takes one numeric parameter named "enabled"
//!		static SML::PARM PARMS_SetEnabled[] = {
//!			{ "enabled", STYPE_Num },
//!			{ 0 }
//!			};
//!
//!		s_AutoClass.AddMethod("IsEnabled",	SIG_b,   (VOIDMETHOD)&FOOBAR::IsEnabled,  NULL);
//!		s_AutoClass.AddMethod("SetEnabled", SIG_v_b, (VOIDMETHOD)&FOOBAR::SetEnabled, PARMS_SetEnabled);
//!		s_AutoClass.AddMethod("SetFocus",	SIG_v,   (VOIDMETHOD)&FOOBAR::SetFocus,   NULL);
//!		}
//!	@endcode
//!
//!	Note:  You cannot use this class directly.  Instead, you should use one of the following
//! template methods:
//! 	<ul>
//!		<li>SML::AUTOCLASS_NEW_T			Uses new/delete to allocate/free the class
//!		<li>SML::AUTOCLASS_NEW_NOCOPY_T		Uses new/delete to allocate/free the class, but no copy constructor
//!		<li>SML::AUTOCLASS_NEW_NOCOPY_T		Uses new/delete to allocate/free the class, but no copy constructor
//!		<li>SML::AUTOCLASS_NOALLOC_T		The class cannot be allocated by iteself.  Used for base classes which cannot be allocated.  Also used for any class which wants to provide its own ctor, dtor, etc.
//!		</ul>
class RVCSYSLIBCLASSEXPORT AUTOCLASS {
public:

	//! Constructor.
	//! Every SML::AUTOCLASS should have one static instance.  When this instance
	//! is constructed, it registers the class so that any SML::CONTEXT created
	//! will know about it.  
	DEPRECATED AUTOCLASS (
		void (*SetupFunc)(),							//! Function to call to setup this class.  Normally a static method of the class we're representing
		CONTEXTTYPE ContextTypesToAllow,			//!< SML::CONTEXTTYPEs to install this class for
		CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< SML::CONTEXTTYPEs to NOT install this class for
		) :
		m_bIsSetup(false),
		m_bIsRegistered(false),
		m_SetupFunc(SetupFunc),
		m_ContextType((CONTEXTTYPE)(ContextTypesToAllow & ~ContextTypesToDisallow))
		{
		::memset(&m_class, 0, sizeof(CLASS));
		m_class.sizeofSMLCLASS = sizeof(CLASS);
		m_class.sizeofSMLCLASSMEMBER = sizeof(CLASSMEMBER);
		m_class.m_Restriction = &m_Restriction;
		m_class.size = 0;
		m_class.Constructor = NULL;
		m_class.Destructor = NULL;
		m_class.CopyConstructor = NULL;
		m_class.GenerateBuilderTextFunc = NULL;
		RegisterSelf();
		}

	//! Constructor.
	//! Every SML::AUTOCLASS should have one static instance.  When this instance
	//! is constructed, it registers the class so that any SML::CONTEXT created
	//! will know about it.  
	AUTOCLASS (
		void (*SetupFunc)()							//! Function to call to setup this class.  Normally a static method of the class we're representing
		) :
		m_bIsSetup(false),
		m_bIsRegistered(false),
		m_SetupFunc(SetupFunc),
		m_ContextType(CONTEXTTYPE_ANY)
		{
		::memset(&m_class, 0, sizeof(CLASS));
		m_class.sizeofSMLCLASS = sizeof(CLASS);
		m_class.sizeofSMLCLASSMEMBER = sizeof(CLASSMEMBER);
		m_class.m_Restriction = &m_Restriction;
		m_class.size = 0;
		m_class.Constructor = NULL;
		m_class.Destructor = NULL;
		m_class.CopyConstructor = NULL;
		m_class.GenerateBuilderTextFunc = NULL;
		RegisterSelf();
		}

	//! Destructor
	virtual ~AUTOCLASS();

	//! Add an old-style method (returning an SML::HASH* (allocated))
	//! This function can return an SML::HASH (which acts like an array) of
	//! classes.  
	void AddMethodOld (
		const char* name,			//!< Method name
		const char* RetClass,		//!< Name of the class held in the SML::HASH (use SMLCLASSNAME macros defined in sml.h if possible)
		CLASSHASH* (*func)(int, ARG*, UINT8*, CONTEXT*),	//<! Function pointer
		PARM* parms,				//!< Parameter list (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bReturnIsAllocated = true, //!< Pass false if the hash should not be freed
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add an old-style method (returning void*)
	void AddMethodOld (
		const char* name,				//!< Name of the method
		const char* RetClass,		//!< Name of the class returned (use SMLCLASSNAME macros defined in sml.h if possible)
		bool bReturnIsAllocated,	//!< Pass true if the return value is allocated so SML will know to free it.  Pass false if SML should not free the reutrn value
		void* (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameters (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add an old-style method that contains a callback(returning void*)
	void AddMethodOldCB (
		const char* name,				//!< Name of the method
		const char* RetClass,		//!< Name of the class returned (use SMLCLASSNAME macros defined in sml.h if possible)
		bool bReturnIsAllocated,	//!< Pass true if the return value is allocated so SML will know to free it.  Pass false if SML should not free the reutrn value
		void* (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameters (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add an old-style method (returning void)
	void AddMethodOld (
		const char* name,				//!< Method name
		void (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameter list (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add an old-style method (returning void)
	//! Special case for when one parameter is a function with its own parameter
	//! list.  The callback's parameter list is expected to follow the main
	//! functions's parameter list (with a null-parameter between them).  This
	//! allows the documentation to document the callback's prototype.
	void AddMethodOldCB (
		const char* name,				//!< Method name
		void (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameter list (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);


	//! Add an old-style method (returning SML::STRING)
	void AddMethodOld (
		const char* name,				//!< Method name
		STRING* (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameter list (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add an old-style method (returning a value)
	void AddMethodOld (
		const char* name,				//!< Method name
		double (*func)(int, ARG*, UINT8*, CONTEXT*),	//!< Pointer to function
		PARM* parms,				//!< Parameter list (can be NULL if none needed)
		int CreateDate = 0,			//!< No longer necessary 
		int ModDate = 0,				//!< No longer necessary
		bool bInWin32Native = true,	//!< Pass false if the method does not exist in Win32Native builds
		bool bInX = true,				//!< Pass false if the method does not exist in X builds
		bool bRequired = false		//!< Set to true to mark method as required
		);

	//! Add a single SML::CLASSMEMBER to the class.  
	//! This should only be used for members, not methods.
	void AddMember (
		const CLASSMEMBER& member
		);

	//! Add an array of SML::CLASSMEMBERs.  The last entry must be blank to single
	//! the end of the array.
	//! This should only be used for members, not methods.
	void AddMembers (
		const CLASSMEMBER* members
		);

	//! Initialize the class
	void Init (
		const char* ClassName,				//!< The name of this class
		const char* BaseClassName = 0,	//!< The name of the base class (NULL if none)
		int CreateDate = 0					//!< Date the class was created (YYYYMMDD format)
		);

	//! Called to create SMLBuilder text for class
	void GenerateBuilderText (
		MISTRING& code, 
		const MISTRING& name, 
		const MISTRINGLIST& ParamList
		) { m_class.GenerateBuilderTextFunc(code, name, ParamList); }

	//! Called to create SMLBuilder code for class
	void SetGenerateBuilderTextFunc (
		void (*GenerateBuilderTextFunc)(MISTRING& code, const MISTRING& name, const MISTRINGLIST& ParamList)
		);

	//! Call this to set a custom constructor/destructor.
	//! By default, the class will be allocated and freed with new/delete.
	//!
	//! There are a couple other other options, however.
	//!
	//!	<b>Option 1</b>: SML lets you allocate and free the structure
	//!		To do this, call SetConstructor and pass pointers to
	//!		constructor and destructor methods.  In this case,
	//!		you can ignore all the parameters in the constructor.
	//!		Allocate your data and return it.
	//!		The destructor can free the data.
	//!		You must not pass a size to SetConstructor() if you want
	//!		SML to let you allocate the memory.
	//!
	//! <b>Option 2</b>: SML Allocates and Frees the structure using its own
	//!		memory pool but lets you initialize and clean up.  To do
	//!		this, pass a non-zero size to SetConstructor().  You can
	//!		also pass pointers to constructor/destructor functions, but
	//!		in this case, the data parameter is already allocated for
	//!		you.  Your constructor can set things up and you destructor
	//!		will be called just before the data is freed.  Your destructor
	//!		must not free the pointer itself.  If there is nothing to do
	//!		to constructo the class (i.e., memseting it to 0 is enough),
	//!		you can pass NULL for the constructor.
	//!
	//! This method is normally only called by the derived templates.
	void SetConstructor (
		void* (*constructor)(void* data, CONTEXT* context, CLASS* smlclass),
		int size = 0
		);

	//! Set the copy constructor
	//! Normally only called by the derived templates
	void SetCopyConstructor (
		void* (*copyconstructor)(void* data, CONTEXT* context, CLASS* smlclass, void*)
		);

	//! Set the class as "deprecated".
	void SetDeprecated (
		) { m_class.Deprecated = 1; }

	void SetDestructor (
		void* (*destructor)(void* data, CONTEXT* context, CLASS* smlclass)
		);

	//! Indicate that the class does not exist in WIN32_NATIVE builds.
	//! This should be called in the Setup method on the X build if the class
	//! does not exist for WIN32_NATIVE builds.  This is used to document this
	//! fact in the documentation when the user is running under X.
	void SetNoWin32Native() {
		m_class.NotInWin32Native = true;
		}

	//! Set the size of an instance of this class.
	//! SML uses this when storing arrays of classes to determine if it can
	//! store them like a SIMPLE_ARRAY<> or if it needs to store an array of
	//! pointers to individually allocated instances.
	void SetSize (
		int size
		) { m_class.size = size; }

#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Called by context creation to add auto-registered classes
	static int InstallAutoClasses (
		CONTEXT* context,
		CONTEXTTYPE contexttype
		);
#endif


	// Lookup a class member of an autoclass without having a context.
	// This is used by the GUI controls so that the XML parsing can lookup
	// the parameters of a callback
	static const CLASSMEMBER* FindMember (
		const char* ClassName,
		const char* MemberName
		);

	//! Get a reference to the class's restrictions
	//! This allows you to modify them
	RESTRICTION& GetRestrictions (
		) { return (m_Restriction); }

protected:
#ifndef GENERATING_DOXYGEN_OUTPUT
//!
//!Only usable by the templates defined below

	virtual void SetNewAndDelete() = 0;

	void AddMethodLow (
		const char* name,
		SMLFUNCSIG sig,
		METHODPTR func,
		PARM* parms,
		int CreateDate = 0,
		int ModDate = 0,
		const char* ClassName = 0,	//!< Name of return class (if returning a class pointer)
		bool bInWin32Native = true,
		bool bInX = true,
		bool bRequired = false
		);

	//! Set the minimum product required to use this class
	void SetMinimumProduct (
		MIPRODUCTCODE ProductCode
		) { m_Restriction.AddProduct(ProductCode); }

private:
	void RegisterSelf();
	void FinishSetup();

	void (*m_SetupFunc)();
	SIMPLE_ARRAY<CLASSMEMBER> m_members;
	SIMPLE_ARRAY<FUNC> m_methods;
	CONTEXTTYPE m_ContextType;
	bool m_bIsSetup;
	bool m_bIsRegistered;
	CLASS m_class;
	RESTRICTION m_Restriction;
#endif // GENERATING_DOXYGEN_OUTPUT
	};


/***************************************************************************/

//! A Template class of SML::AUTOCLASS.  See documentation on SML::AUTOCLASS
//! Use this template for classes which are pure-virtual base classes which
//! cannot be allocated with new and delete, and which cannot actually be
//! allocated at all.
//! You can also use this class and provide your own ctor, dtor and copy
//! constructor
template <class _TYPE> class AUTOCLASS_NOALLOC_T : public AUTOCLASS {
	public:

		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NOALLOC_T (
			void (*SetupFunc)()					//! Function to call to setup this class.  Normally a static method of the class we're representing
			) :
			AUTOCLASS (SetupFunc)
			{
			}

		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NOALLOC_T (
			void (*SetupFunc)(),					//! Function to call to setup this class.  Normally a static method of the class we're representing
			CONTEXTTYPE ContextTypesToAllow,		//!< SML::CONTEXTTYPEs to install this class for
			CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< CONTEXTTYPEs to NOT install this class for
			) :
			AUTOCLASS (SetupFunc, ContextTypesToAllow, ContextTypesToDisallow)
			{
			}

		typedef void (_TYPE::*VOIDMETHOD)();

		//! Add a method.
		//!
		//! This is the main bit of magic.  It allows you to define a method and
		//! have SML use the C++ implementation directly rather than having to
		//! write a wrapper around it.
		//!
		//! This can only be used for methods that take up to 3 parameters, and
		//! have no overloaded versions in C++.  You also cannot use this to
		//! define a method with parameters that are optional in SML. (The C++
		//! implementation can have optional parameters, but SML will always
		//! pass a value).
		//!
		//! The sig parameter tells the return type and the actual C++ type of
		//! the parameters.  The first letter indicates the return type and the
		//! following letters indicate the parameter types.  Possible values are:
		//!	<ul>
		//!	<li><b>v</b> - void (return type only)
		//!	<li><b>b</b> - bool 
		//!	<li><b>d</b> - double 
		//!	<li><b>i</b> - INT32 
		//!	<li><b>u</b> - UINT32 
		//!	<li><b>pv</b> - (void*) Note, can also be used for references
		//!	<li><b>pva</b> - (void*) Same as pv, but tells SML it needs to free the return value (only used as a return type)
		//!	<li><b>U</b> - (MIUNICODE*) If used as a return type, SML will free the string when it's done with it
		//!	<li><b>cU</b> - (const MIUNICODE*) 
		//!	<li><b>c</b> - (char*) If used as a return type, SML will free the string when it's done with it
		//!	<li><b>cc</b> - (const char*) 
		//!	</ul>
		//! NOTE: you can pass an MISTRING& by using pv in the SIG_ and STYPE_String
		//! in the SML::PARM array for the parameter.
		void AddMethod (
			const char* name,				//!< Name of the method
			SMLFUNCSIG sig,				//!< Example, SIG_v_i == function taking an int and returning nothing (void)
			void (_TYPE::*func)(),			//! parameters for the method (can be NULL if no parms)
			PARM* parms,				//!< Parameters.  Number and kind must match signature or else...
			int CreateDate = 0,				//!< The date this method was added (YYYYMMDD format)
			int ModDate = 0,				//!< The date this method was changed (0 if not changed since added)
			const char* ClassName = 0,	//!< Name of return class (if returning a class pointer)
			bool bInWin32Native = true,	//!< Set to false if method not available in WIN32_NATIVE code
			bool bInX = true,					//!< Set to false if method not available in X_NATIVE code
			bool bRequired = false		//!< Set to true to mark method as required
			) {
			union {
				void (_TYPE::*cfunc)();
				METHODPTR vfunc;
				} x;
			::memset(&x, 0, sizeof(x));
			x.cfunc = func;
			AddMethodLow(name, sig, x.vfunc, parms, CreateDate, ModDate, ClassName, bInWin32Native, bInX, bRequired);
			}

#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		//! Called by Init to set the default allocation scheme
		//! Does nothing in this case since class has pure virtuals
		virtual void SetNewAndDelete () {
			}
#endif

	};


//! A Template class of SML::AUTOCLASS.  See documentation on SML::AUTOCLASS
//! This is the same as SML::AUTOCLASS_T, but the class will be allocated
//! with new and delete.
template <class _TYPE> class AUTOCLASS_NEW_T : public AUTOCLASS_NOALLOC_T<_TYPE> {
	public:
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NEW_T (
			void (*SetupFunc)()					//! Function to call to setup this class.  Normally a static method of the class we're representing
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc)
			{
			}
			
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NEW_T (
			void (*SetupFunc)(),					//! Function to call to setup this class.  Normally a static method of the class we're representing
			CONTEXTTYPE ContextTypesToAllow,		//!< SML::CONTEXTTYPEs to install this class for
			CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< SML::CONTEXTTYPEs to NOT install this class for
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc, ContextTypesToAllow, ContextTypesToDisallow)
			{
			}

#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		//! Called by Init to set the default behaviour of using new/delete
		virtual void SetNewAndDelete (
			) {
			SetConstructor(Constructor);
			SetDestructor(Destructor);
			SetCopyConstructor(CopyConstructor);
			}

		static void* Constructor(void*, CONTEXT*, CLASS*) {
			return new _TYPE;
			}

		static void* Destructor(void* vptr, CONTEXT*, CLASS*) {
			_TYPE* ptr = static_cast<_TYPE*>(vptr);
			delete ptr;
			return 0;
			}

		static void* CopyConstructor(void* vrhs, CONTEXT*, CLASS*, void*vdest) {
			_TYPE* rhs = static_cast<_TYPE*>(vrhs);
			if (vdest != 0) {
				_TYPE* dest = static_cast<_TYPE*>(vdest);
				if (rhs == 0) *dest = _TYPE();
				else *dest = *rhs;
				return dest;
				}
			if (rhs == 0) return new _TYPE;
			return new _TYPE(*rhs);
			}
#endif
	};

//! A Template class of SML::AUTOCLASS.  See documentation on SML::AUTOCLASS
//! This is the same as SML::AUTOCLASS_NEW_T, but for classes which have no
//! copy constructor.
template <class _TYPE> class AUTOCLASS_NEW_NOCOPY_T : public AUTOCLASS_NOALLOC_T<_TYPE> {
	public:
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NEW_NOCOPY_T (
			void (*SetupFunc)()					//! Function to call to setup this class.  Normally a static method of the class we're representing
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc)
			{
			}
	
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_NEW_NOCOPY_T (
			void (*SetupFunc)(),					//! Function to call to setup this class.  Normally a static method of the class we're representing
			CONTEXTTYPE ContextTypesToAllow,		//!< SML::CONTEXTTYPEs to install this class for
			CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< SML::CONTEXTTYPEs to NOT install this class for
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc, ContextTypesToAllow, ContextTypesToDisallow)
			{
			}

#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		virtual void SetNewAndDelete (
			) {
			SetConstructor(Constructor);
			SetDestructor(Destructor);
			}

		static void* Constructor(void*, CONTEXT*, CLASS*) {
			return new _TYPE;
			}

		static void* Destructor(void* vptr, CONTEXT*, CLASS*) {
			_TYPE* ptr = static_cast<_TYPE*>(vptr);
			delete ptr;
			return 0;
			}
#endif

	};

//! A Template class of SML::AUTOCLASS.  See documentation on SML::AUTOCLASS
//! This is the same as SML::AUTOCLASS_DELETABLE_T, but for classes which have
//! no copy constructor.
template <class _TYPE> class AUTOCLASS_DELETABLE_NOCOPY_T : public AUTOCLASS_NOALLOC_T<_TYPE> {
	public:
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_DELETABLE_NOCOPY_T (
			void (*SetupFunc)()					//! Function to call to setup this class.  Normally a static method of the class we're representing
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc)
			{
			}

		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_DELETABLE_NOCOPY_T (
			void (*SetupFunc)(),					//! Function to call to setup this class.  Normally a static method of the class we're representing
			CONTEXTTYPE ContextTypesToAllow,		//!< SML::CONTEXTTYPEs to install this class for
			CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< CONTEXTTYPEs to NOT install this class for
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc, ContextTypesToAllow, ContextTypesToDisallow)
			{
			}
#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		virtual void SetNewAndDelete (
			) {
			SetDestructor(Destructor);
			}

		static void* Destructor(void* vptr, CONTEXT*, CLASS*) {
			_TYPE* ptr = static_cast<_TYPE*>(vptr);
			delete ptr;
			return 0;
			}

#endif

	};

//! A Template class of SML::AUTOCLASS.  See documentation on SML::AUTOCLASS
//! This is the same as SML::AUTOCLASS_NEW_T, but for classes which have
//! no default constructor.  Use this if the class must be allocated by calling
//! some function but should be deleted with a normal "delete"
template <class _TYPE> class AUTOCLASS_DELETABLE_T : public AUTOCLASS_NOALLOC_T<_TYPE> {
	public:
		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_DELETABLE_T (
			void (*SetupFunc)()					//! Function to call to setup this class.  Normally a static method of the class we're representing
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc)
			{
			}

		//! Constructor.
		//! Every SML::AUTOCLASS should have one static instance.  When this instance
		//! is constructed, it registers the class so that any SML::CONTEXT created
		//! will know about it.  
		AUTOCLASS_DELETABLE_T (
			void (*SetupFunc)(),					//! Function to call to setup this class.  Normally a static method of the class we're representing
			CONTEXTTYPE ContextTypesToAllow,		//!< SML::CONTEXTTYPEs to install this class for
			CONTEXTTYPE ContextTypesToDisallow = CONTEXTTYPE_NONE		//!< SML::CONTEXTTYPEs to NOT install this class for
			) :
			AUTOCLASS_NOALLOC_T<_TYPE>(SetupFunc, ContextTypesToAllow, ContextTypesToDisallow)
			{
			}

#ifndef GENERATING_DOXYGEN_OUTPUT
	protected:

		virtual void SetNewAndDelete (
			) {
			SetDestructor(Destructor);
			SetCopyConstructor(CopyConstructor);
			}

		static void* Destructor(void* vptr, CONTEXT*, CLASS*) {
			_TYPE* ptr = static_cast<_TYPE*>(vptr);
			delete ptr;
			return 0;
			}

		static void* CopyConstructor(void* vrhs, CONTEXT*, CLASS*, void*vdest) {
			_TYPE* rhs = static_cast<_TYPE*>(vrhs);
			if (vdest != 0) {
				_TYPE* dest = static_cast<_TYPE*>(vdest);
				if (rhs == 0) return (NULL); // Bad, no default constructor
				else *dest = *rhs;
				return dest;
				}
			if (rhs == 0) return (NULL);  // Bad, no default constructor
			return new _TYPE(*rhs);
			}
#endif

	};

} // end of SML namespace


typedef SML::CLASS SMLCLASS;
typedef SML::AUTOCLASS SMLAUTOCLASS;

#define SMLAUTOCLASS_NOALLOC_T SML::AUTOCLASS_NOALLOC_T
#define SMLAUTOCLASS_NEW_T SML::AUTOCLASS_NEW_T
#define SMLAUTOCLASS_NEW_NOCOPY_T SML::AUTOCLASS_NEW_NOCOPY_T
#define SMLAUTOCLASS_DELETABLE_NOCOPY_T SML::AUTOCLASS_DELETABLE_NOCOPY_T
#define SMLAUTOCLASS_DELETABLE_T SML::AUTOCLASS_DELETABLE_T

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_CLASS_H


