/******************************************************************************
 *
 * \file sml/sml.h
 * \brief SML  user data
 *
 * \if NODOC
 * $Log: userdata.h_v $
 * Revision 1.3  2011/10/12 14:33:52  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2007/05/08 15:46:38  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 20:44:40  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_USERDATA_H
#define INC_SML_USERDATA_H

#ifndef INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace SML {
/***************************************************************************/

//!	\class SML::USERDATA sml/userdata.h
//!	A class which can be used to hang USERDATA off an SML::STRING or SML::SYMBOL
//!
//!	To use, you have to derive a class from SML::USERDATA,
//!	Create your own ID in the enum below, and make your constructor
//! call the base class constructor, passing in your id.
//!
//!	For MicroImages employees adding derived classes, just use the
//! next available number in the TAG enum.   If using TNTsdk,
//! start your numbers at something high like 1024 so there's no
//! chance of conflicting with later additions by MicroImages.
//!
//!
class RVCSYSLIBCLASSEXPORT USERDATA {
	public:

	enum TAG {
		TAG_NONE			= 0,
		TAG_TABLE		= 1,
		TAG_FIELD		= 2,
		TAG_REGEXP		= 3,
		TAG_FirstUserTag		= 1000,	//!< The first tag that AllocTag will return
		TAG_BIGNUMBER	= 0x7FFFFFF	//!< Make compiler use a long, not a byte
		};

	USERDATA (
		TAG tag
		) :
		m_tag(tag)
		{
		}

	virtual ~USERDATA (
		) {
		}

	//! Allocate a tag to use for a user-defined SML::USERDATA
	//! Note: You should allocate one and keep it allocated.
	//! 	unless you need to attach multiple instances of the
	//! 	same type
	static TAG AllocTag();

	TAG GetTag (
		) const {
		return m_tag;
		}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TAG m_tag;
		static TAG s_NextTag;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#ifndef GENERATING_DOXYGEN_OUTPUT
class USERDATACONTAINER {
	public:
	USERDATACONTAINER() : m_userdata(0) {}
	~USERDATACONTAINER();
	void Init() { m_userdata = 0;	}		//! Used if enclosing class has no constructor
	void Destroy();		//! Used if enclosing class has no destructor

	//! Attach some User Data
	//! The data to be attached must be derived from SML::USERDATA, and must have
	//!	been allocated via "new".  Once attached, the container owns the
	//! data and will "delete" it when the container is deleted.
	//! Also, if you try to attach something for a user ID that's already in use,
	//! it will delete the existing thing.
	void Attach (
		USERDATA*
		);

	//! Unattach some User Data.
	//! If the data to be unattached isn't attached, it just ignores you.
	//! Once unattached, the data is yours to do with as you wish.  You are
	//! responsible for deleting it too.
	void Detach (
		USERDATA*
		);


	//! Get the User Data.
	//! If no data for the given id is attached, it will just return NULL	.
	//! Do not free the data returned unless you Unattach it first.
	USERDATA* Get (
		USERDATA::TAG
		);

	private:
		SIMPLE_ARRAY<USERDATA*>* m_userdata;	//!< A pointer because it's only allocateed if needed (almost never)
	};

#endif // GENERATING_DOXYGEN_OUTPUT
} // end of SML namespace

typedef SML::USERDATA SMLUSERDATA;
typedef SML::USERDATACONTAINER SMLUSERDATACONTAINER;


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_USERDATA_H


