/**
 * \file imagefiltermosaic.h <spatmod/imagefiltermosaic.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_MOSAIC definition.
 *
 * \if NODOC
 * $Id: imagefiltermosaic.h_v 1.4 2009/04/10 16:32:39 mju Exp $
 *
 * $Log: imagefiltermosaic.h_v $
 * Revision 1.4  2009/04/10 16:32:39  mju
 * Just keep source extent rectangles, not regions.
 *
 * Revision 1.3  2007/07/24 17:47:26  mju
 * Use LRU list for inputs.
 *
 * Revision 1.2  2007/07/18 17:16:17  mju
 * Add feathermethod enum.
 *
 * Revision 1.1  2007/03/26 13:13:08  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERMOSAIC_H
#define	INC_SPATMOD_IMAGEFILTERMOSAIC_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

class REGION2D;

namespace SPATMOD {
namespace IMAGE {

class BUFFER;
class SAMPLEITERATOR;
class SAMPLEITERATOR_CONST;

//=====================================================================================================================

//! Filter to mosaic images.
//! The coordinate transformation between resulting image and each input stage is computed using georeference as
//! specified in constructor, or georeference from specified reference image, along with the georeference
//! from the input stage.  An optional transformation accuracy may be specified via SetAccuracy() to allow use
//! of efficient affine transformation if appropriate.
//! Note that source images will not be processed "sequentially", and the sections specified
//! in PullBegin for the source will overlap.  It is recommended that a suitable caching filter be placed ahead
//! of this filter if the input requires significant computation or sequential iteration.
class FILTER_MOSAIC : public FILTER {
	public:

		enum FEATHERMETHOD {
			FEATHERMETHOD_None,
			FEATHERMETHOD_Linear,
			FEATHERMETHOD_Nonlinear
			};

		enum OVERLAPMETHOD {
			OVERLAPMETHOD_First,						//!< Use first valid pixel
			OVERLAPMETHOD_Last,						//!< Use last valid pixel
			OVERLAPMETHOD_Minimum,					//!< Use pixel with minimum total of sample values
			OVERLAPMETHOD_Maximum,					//!< Use pixel with maximum total of sample values
			OVERLAPMETHOD_ClosestToSpecified		//!< Use pixel closest to specified value
			};

		//! Construct using specified dimensions and georeference.
		FILTER_MOSAIC (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages, all must have same pixel properties and be georeferenced
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			const DIMENSIONS& Dimensions,			//!< Resulting image dimensions
			const GEOREFERENCE& Georeference,	//!< Georeference for resulting image
			RESAMPLEMETHOD ResampleMethod,		//!< Resampling method
			OVERLAPMETHOD OverlapMethod			//!< Overlap control method
			);

		//! Construct to match specified reference image.
		FILTER_MOSAIC (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages, all must have same pixel type and be georeferenced
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			const BASE& RefImage,					//!< Reference image to match
			RESAMPLEMETHOD ResampleMethod,		//!< Resampling method
			OVERLAPMETHOD OverlapMethod			//!< Overlap control method
			);

		// Destructor.
		virtual ~FILTER_MOSAIC ();

		//! Set pixel for OVERLAPMETHOD_ClosestToSpecified.
		//! If not specified, the center of the valid data range will be used by default.
		ERRVALUE SetPixelTestClosest (
			const PIXEL& PixelTestClosest			//!< Pixel value, must have same properties as image itself
			);

		//! Set transformation accuracy in source cells.
		//! If within the specified accuracy, a fast affine transformation will be used for computation.
		//! If used, must be done before Initialize().
		//! Default accuracy is .5 cells.
		void SetAccuracy (
			double accuracy							//!< Transformation accuracy in source cells.
			) { m_Accuracy = accuracy; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class INPUTLIST;

		RESAMPLEMETHOD m_ResampleMethod;
		OVERLAPMETHOD m_OverlapMethod;
		double m_Accuracy;
		PIXEL m_PixelTestClosest;
		INT32 m_MaxColumnsPull;
		INT32 m_MaxRowsPull;
		double *m_dValueTestClosest;
		DRECT2D *m_ExtentsInputInOutput;
		SAMPLEITERATOR_CONST *m_itInput;
		SAMPLEITERATOR *m_itOutput;
		INPUTLIST *m_pInputListLRU;
		int m_MaxListSizeLRU;

		void InitPixelTestClosest ();
		ERRVALUE ProcessOverlap (BUFFER& BufferOutput, const BUFFER& BufferInput, INT32 NumColumns, INT32 NumRows) const;
		ERRVALUE RemoveLastFromLRU ();

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_MOSAIC (const FILTER_MOSAIC&);
		FILTER_MOSAIC& operator= (const FILTER_MOSAIC&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERMOSAIC_H

