/**
 * \file imagepixeltable.h <spatmod/imagepixeltable.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SAMPLETABLE class definition
 *
 * \if NODOC
 * $Id: imagesampletable.h_v 1.5 2007/01/02 21:54:28 mju Exp $
 *
 * $Log: imagesampletable.h_v $
 * Revision 1.5  2007/01/02 21:54:28  mju
 * Add non-const version of getPtrItems.
 *
 * Revision 1.4  2007/01/02 18:54:58  mju
 * Add exchange() method.
 *
 * Revision 1.3  2006/11/17 16:50:06  mju
 * Add getPtrItem.
 *
 * Revision 1.2  2006/01/30 21:41:17  mju
 * Add setItem using double.
 *
 * Revision 1.1  2006/01/30 17:40:33  mju
 * Initial revision
 *
 * Revision 1.2  2005/12/15 18:08:25  mju
 * Add getColors.
 *
 * Revision 1.1  2005/08/16 15:10:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESAMPLETABLE_H
#define	INC_SPATMOD_IMAGESAMPLETABLE_H

#ifndef	INC_SPATMOD_IMAGESAMPLE_H
#include <spatmod/imagesample.h>
#endif

#ifndef	INC_MI32_MIDBLARY_H
#include	<mi32/midblary.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Image sample table.
class SAMPLETABLE {
	public:

		//! Default constructor.
		SAMPLETABLE ();

		//! Copy constructor
		SAMPLETABLE (
			const SAMPLETABLE& rhs
			);

		//! Destructor.
		~SAMPLETABLE ();

		//! Assignment.
		SAMPLETABLE& operator= (
			const SAMPLETABLE& rhs
			);

		//! Create sample table.
		//! Will set all items to value specified by properties if no pValues array provided.
		ERRVALUE Create (
			const SAMPLE& SampleProperties,	//!< Sample properties
			UINT32 NumItems,						//!< Number of table items
			const void *pValues = 0				//!< Array of values, 0 to not set
			);

		//! Exchange data with another sample table.
		void Exchange (
			SAMPLETABLE& other					//!< Table to exchange with
			);

		//! Get number of items in table.
		UINT32 GetNumItems (
			) const { return (m_NumItems); }

		//! Get pointer to single item.
		//! For efficiency should not be used in pixel processing loops.
		const void* GetPtrItem (
			UINT32 ItemIndex
			) const { return (static_cast<const UINT8*>(GetPtrItems()) + ItemIndex * m_SampleProperties.GetNumBytes()); }

		//! Get pointer to table items.
		const void* GetPtrItems (
			) const { return (static_cast<const void*>(m_Values.GetPointer())); }

		//! Get pointer to table items (non-const).
		void* GetPtrItems (
			) { return (static_cast<void*>(m_Values.GetPointer())); }

		//! Get sample properties.
		const SAMPLE& GetSampleProperties (
			) const { return (m_SampleProperties); }

		//! Set all items to same value.
		ERRVALUE SetAllItems (
			const SAMPLE& value					//!< Sample value to set
			);

		//! Set single table item with index and property validation.
		ERRVALUE SetItem (
			UINT32 index,							//!< Index in table
			const SAMPLE& value					//!< Sample value to set
			);

		//! Set single table item with index and range validation.
		ERRVALUE SetItem (
			UINT32 index,							//!< Index in table
			double value							//!< Value to sets
			);

		//! Set table items.
		ERRVALUE SetItems (
			UINT32 StartIndex,					//!< Starting index in table
			const void *pValues,					//!< Pointer to sample values to set, must be correct type
			UINT32 NumItems = 1					//!< Number of items to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SAMPLE m_SampleProperties;
		MIDOUBLEARRAY m_Values;
		UINT32 m_NumItems;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGESAMPLETABLE_H
